@@ -1,24 +1,19 @@
 /*
- * Copyright (C) 2004-2014  Internet Systems Consortium, Inc. ("ISC")
- * Copyright (C) 2001-2003  Internet Software Consortium.
+ * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
- * Permission to use, copy, modify, and/or distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
- * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
- * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
- * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
- * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
- * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
+ * See the COPYRIGHT file distributed with this work for additional
+ * information regarding copyright ownership.
  */
 
 /*! \file */
 
 #include <config.h>
 
+#include <inttypes.h>
 #include <stdlib.h>
 
 #include <isc/buffer.h>
@@ -31,6 +26,9 @@
 #include <isc/string.h>
 #include <isc/util.h>
 
+#include <pk11/site.h>
+
+#include <isccfg/grammar.h>
 #include <isccfg/namedconf.h>
 
 #include <dns/fixedname.h>
@@ -50,184 +48,216 @@
 /*% default configuration */
 static char defaultconf[] = "\
 options {\n\
+	answer-cookie true;\n\
 	automatic-interface-scan yes;\n\
+	bindkeys-file \"" NAMED_SYSCONFDIR "/bind.keys\";\n\
 #	blackhole {none;};\n"
+#if defined(HAVE_OPENSSL_AES) || defined(HAVE_OPENSSL_EVP_AES)
+"	cookie-algorithm aes;\n"
+#else
+"	cookie-algorithm sha256;\n"
+#endif
 #ifndef WIN32
 "	coresize default;\n\
-	datasize default;\n\
-	files unlimited;\n\
-	stacksize default;\n"
+	datasize default;\n"
 #endif
-"#	session-keyfile \"" NS_LOCALSTATEDIR "/run/named/session.key\";\n\
-	session-keyname local-ddns;\n\
-	session-keyalg hmac-sha256;\n\
-	deallocate-on-exit true;\n\
+"\
+#	deallocate-on-exit <obsolete>;\n\
 #	directory <none>\n\
 	dump-file \"named_dump.db\";\n\
-	fake-iquery no;\n\
-	has-old-clients false;\n\
+	edns-udp-size 4096;\n\
+#	fake-iquery <obsolete>;\n"
+#ifndef WIN32
+"	files unlimited;\n"
+#endif
+"\
+#	has-old-clients <obsolete>;\n\
 	heartbeat-interval 60;\n\
-	host-statistics no;\n\
+#	host-statistics <obsolete>;\n\
 	interface-interval 60;\n\
+#	keep-response-order {none;};\n\
 	listen-on {any;};\n\
 	listen-on-v6 {any;};\n\
+#	lock-file \"" NAMED_LOCALSTATEDIR "/run/named/named.lock\";\n\
 	match-mapped-addresses no;\n\
 	max-rsa-exponent-size 0; /* no limit */\n\
-	memstatistics-file \"named.memstats\";\n\
-	multiple-cnames no;\n\
-#	named-xfer <obsolete>;\n\
-#	pid-file \"" NS_LOCALSTATEDIR "/run/named/named.pid\"; /* or /lwresd.pid */\n\
-	bindkeys-file \"" NS_SYSCONFDIR "/bind.keys\";\n\
-	port 53;\n\
-	prefetch 2 9;\n\
-	recursing-file \"named.recursing\";\n\
-	secroots-file \"named.secroots\";\n\
-"
-#ifdef PATH_RANDOMDEV
-"\
-	random-device \"" PATH_RANDOMDEV "\";\n\
-"
-#endif
-"\
-	recursive-clients 1000;\n\
-	resolver-query-timeout 10;\n\
-	rrset-order { order random; };\n\
-	serial-queries 20;\n\
-	serial-query-rate 20;\n\
-	server-id none;\n\
-	statistics-file \"named.stats\";\n\
-	statistics-interval 60;\n\
-	tcp-clients 100;\n\
-	tcp-listen-queue 10;\n\
-#	tkey-dhkey <none>\n\
-#	tkey-gssapi-credential <none>\n\
-#	tkey-domain <none>\n\
-	transfers-per-ns 2;\n\
-	transfers-in 10;\n\
-	transfers-out 10;\n\
-	treat-cr-as-space true;\n\
-	use-id-pool true;\n\
-	use-ixfr true;\n\
-	edns-udp-size 4096;\n\
 	max-udp-size 4096;\n\
-"
-#ifdef ISC_PLATFORM_USESIT
-"\
-	nosit-udp-size 4096;\n\
-	request-sit true;\n\
-"
+	memstatistics-file \"named.memstats\";\n\
+#	multiple-cnames <obsolete>;\n\
+#	named-xfer <obsolete>;\n\
+	nocookie-udp-size 4096;\n\
+	notify-rate 20;\n\
+	nta-lifetime 3600;\n\
+	nta-recheck 300;\n\
+#	pid-file \"" NAMED_LOCALSTATEDIR "/run/named/named.pid\"; \n\
+	port 53;\n\
+	prefetch 2 9;\n"
+#if defined(ISC_PLATFORM_CRYPTORANDOM)
+"	random-device none;\n"
+#elif defined(PATH_RANDOMDEV)
+"	random-device \"" PATH_RANDOMDEV "\";\n"
 #endif
-"\
+"	recursing-file \"named.recursing\";\n\
+	recursive-clients 1000;\n\
 	request-nsid false;\n\
 	reserved-sockets 512;\n\
-\n\
-	/* DLV */\n\
-	dnssec-lookaside . trust-anchor dlv.isc.org;\n\
+	resolver-query-timeout 10;\n\
+	rrset-order { order random; };\n\
+	secroots-file \"named.secroots\";\n\
+	send-cookie true;\n\
+#	serial-queries <obsolete>;\n\
+	serial-query-rate 20;\n\
+	server-id none;\n\
+	session-keyalg hmac-sha256;\n\
+#	session-keyfile \"" NAMED_LOCALSTATEDIR "/run/named/session.key\";\n\
+	session-keyname local-ddns;\n"
+#ifndef WIN32
+"	stacksize default;\n"
+#endif
+"	startup-notify-rate 20;\n\
+	statistics-file \"named.stats\";\n\
+#	statistics-interval <obsolete>;\n\
+	tcp-advertised-timeout 300;\n\
+	tcp-clients 150;\n\
+	tcp-idle-timeout 300;\n\
+	tcp-initial-timeout 300;\n\
+	tcp-keepalive-timeout 300;\n\
+	tcp-listen-queue 10;\n\
+#	tkey-dhkey <none>\n\
+#	tkey-domain <none>\n\
+#	tkey-gssapi-credential <none>\n\
+	transfer-message-size 20480;\n\
+	transfers-in 10;\n\
+	transfers-out 10;\n\
+	transfers-per-ns 2;\n\
+#	treat-cr-as-space <obsolete>;\n\
+	trust-anchor-telemetry yes;\n\
+#	use-id-pool <obsolete>;\n\
+#	use-ixfr <obsolete>;\n\
 \n\
 	/* view */\n\
+	allow-new-zones no;\n\
 	allow-notify {none;};\n\
-	allow-update-forwarding {none;};\n\
 	allow-query-cache { localnets; localhost; };\n\
 	allow-query-cache-on { any; };\n\
 	allow-recursion { localnets; localhost; };\n\
 	allow-recursion-on { any; };\n\
+	allow-update-forwarding {none;};\n\
 #	allow-v6-synthesis <obsolete>;\n\
-#	sortlist <none>\n\
-#	topology <none>\n\
 	auth-nxdomain false;\n\
-	minimal-responses false;\n\
-	recursion true;\n\
-	provide-ixfr true;\n\
-	request-ixfr true;\n\
-	fetch-glue no;\n\
-	rfc2308-type1 no;\n\
-	additional-from-auth true;\n\
-	additional-from-cache true;\n\
-	query-source address *;\n\
-	query-source-v6 address *;\n\
-	notify-source *;\n\
-	notify-source-v6 *;\n\
-	cleaning-interval 0;  /* now meaningless */\n\
-	min-roots 2;\n\
-	lame-ttl 600;\n\
-	max-ncache-ttl 10800; /* 3 hours */\n\
-	max-cache-ttl 604800; /* 1 week */\n\
-	transfer-format many-answers;\n\
-	max-cache-size 0;\n\
-	check-names master fail;\n\
-	check-names slave warn;\n\
-	check-names response ignore;\n\
 	check-dup-records warn;\n\
 	check-mx warn;\n\
+	check-names master fail;\n\
+	check-names response ignore;\n\
+	check-names slave warn;\n\
 	check-spf warn;\n\
-	acache-enable no;\n\
-	acache-cleaning-interval 60;\n\
-	max-acache-size 16M;\n\
-	dnssec-enable yes;\n\
-	dnssec-validation yes; \n\
-	dnssec-accept-expired no;\n\
+	cleaning-interval 0;  /* now meaningless */\n\
 	clients-per-query 10;\n\
-	max-clients-per-query 100;\n\
-	max-recursion-depth 7;\n\
-	max-recursion-queries 50;\n\
-	zero-no-soa-ttl-cache no;\n\
-	nsec3-test-zone no;\n\
-	allow-new-zones no;\n\
-"
-#ifdef ALLOW_FILTER_AAAA
-"	filter-aaaa-on-v4 no;\n\
-	filter-aaaa-on-v6 no;\n\
-	filter-aaaa { any; };\n\
-"
+	dnssec-accept-expired no;\n\
+	dnssec-enable yes;\n\
+	dnssec-validation yes; \n"
+#ifdef HAVE_DNSTAP
+"	dnstap-identity hostname;\n"
 #endif
-
-"	/* zone */\n\
+"\
+#	fetch-glue <obsolete>;\n\
+	fetch-quota-params 100 0.1 0.3 0.7;\n\
+	fetches-per-server 0;\n\
+	fetches-per-zone 0;\n\
+	filter-aaaa-on-v4 no;\n\
+	filter-aaaa-on-v6 no;\n\
+	filter-aaaa { any; };\n"
+#ifdef HAVE_GEOIP
+"	geoip-use-ecs yes;\n"
+#endif
+"	glue-cache yes;\n\
+	lame-ttl 600;\n"
+#ifdef HAVE_LMDB
+"	lmdb-mapsize 32M;\n"
+#endif
+"	max-cache-size 90%;\n\
+	max-cache-ttl 604800; /* 1 week */\n\
+	max-clients-per-query 100;\n\
+	max-ncache-ttl 10800; /* 3 hours */\n\
+	max-recursion-depth 7;\n\
+	max-recursion-queries 75;\n\
+	max-stale-ttl 604800; /* 1 week */\n\
+	message-compression yes;\n\
+#	min-roots <obsolete>;\n\
+	minimal-any false;\n\
+	minimal-responses no-auth-recursive;\n\
+	notify-source *;\n\
+	notify-source-v6 *;\n\
+	nsec3-test-zone no;\n\
+	provide-ixfr true;\n\
+	query-source address *;\n\
+	query-source-v6 address *;\n\
+	recursion true;\n\
+	request-expire true;\n\
+	request-ixfr true;\n\
+	require-server-cookie no;\n\
+	resolver-nonbackoff-tries 3;\n\
+	resolver-retry-interval 800; /* in milliseconds */\n\
+#	rfc2308-type1 <obsolete>;\n\
+	root-key-sentinel yes;\n\
+	servfail-ttl 1;\n\
+#	sortlist <none>\n\
+	stale-answer-enable false;\n\
+	stale-answer-ttl 1; /* 1 second */\n\
+	synth-from-dnssec yes;\n\
+#	topology <none>\n\
+	transfer-format many-answers;\n\
+	v6-bias 50;\n\
+	zero-no-soa-ttl-cache no;\n\
+\n\
+	/* zone */\n\
 	allow-query {any;};\n\
 	allow-query-on {any;};\n\
 	allow-transfer {any;};\n\
-	notify yes;\n\
 #	also-notify <none>\n\
-	notify-delay 5;\n\
-	notify-to-soa no;\n\
-	dialup no;\n\
-#	forward <none>\n\
-#	forwarders <none>\n\
-	maintain-ixfr-base no;\n\
-#	max-ixfr-log-size <obsolete>\n\
-	transfer-source *;\n\
-	transfer-source-v6 *;\n\
 	alt-transfer-source *;\n\
 	alt-transfer-source-v6 *;\n\
-	max-transfer-time-in 120;\n\
-	max-transfer-time-out 120;\n\
+	check-integrity yes;\n\
+	check-mx-cname warn;\n\
+	check-sibling yes;\n\
+	check-srv-cname warn;\n\
+	check-wildcard yes;\n\
+	dialup no;\n\
+	dnssec-dnskey-kskonly no;\n\
+	dnssec-loadkeys-interval 60;\n\
+	dnssec-secure-to-insecure no;\n\
+	dnssec-update-mode maintain;\n\
+#	forward <none>\n\
+#	forwarders <none>\n\
+	inline-signing no;\n\
+	ixfr-from-differences false;\n\
+#	maintain-ixfr-base <obsolete>;\n\
+#	max-ixfr-log-size <obsolete>\n\
+	max-journal-size default;\n\
+	max-records 0;\n\
+	max-refresh-time 2419200; /* 4 weeks */\n\
+	max-retry-time 1209600; /* 2 weeks */\n\
 	max-transfer-idle-in 60;\n\
 	max-transfer-idle-out 60;\n\
-	max-retry-time 1209600; /* 2 weeks */\n\
-	min-retry-time 500;\n\
-	max-refresh-time 2419200; /* 4 weeks */\n\
+	max-transfer-time-in 120;\n\
+	max-transfer-time-out 120;\n\
 	min-refresh-time 300;\n\
+	min-retry-time 500;\n\
 	multi-master no;\n\
-	dnssec-secure-to-insecure no;\n\
-	sig-validity-interval 30; /* days */\n\
+	notify yes;\n\
+	notify-delay 5;\n\
+	notify-to-soa no;\n\
+	serial-update-method increment;\n\
 	sig-signing-nodes 100;\n\
 	sig-signing-signatures 10;\n\
 	sig-signing-type 65534;\n\
-	inline-signing no;\n\
-	zone-statistics terse;\n\
-	max-journal-size unlimited;\n\
-	ixfr-from-differences false;\n\
-	check-wildcard yes;\n\
-	check-sibling yes;\n\
-	check-integrity yes;\n\
-	check-mx-cname warn;\n\
-	check-srv-cname warn;\n\
-	zero-no-soa-ttl yes;\n\
-	update-check-ksk yes;\n\
-	serial-update-method increment;\n\
-	dnssec-update-mode maintain;\n\
-	dnssec-dnskey-kskonly no;\n\
-	dnssec-loadkeys-interval 60;\n\
+	sig-validity-interval 30; /* days */\n\
+	transfer-source *;\n\
+	transfer-source-v6 *;\n\
 	try-tcp-refresh yes; /* BIND 8 compat */\n\
+	update-check-ksk yes;\n\
+	zero-no-soa-ttl yes;\n\
+	zone-statistics terse;\n\
 };\n\
 "
 
@@ -268,8 +298,8 @@ view \"_bind\" chaos {\n\
 };\n\
 "
 "#\n\
-#  Default trusted key(s) for builtin DLV support\n\
-#  (used if \"dnssec-lookaside auto;\" is set and\n\
+#  Default trusted key(s), used if \n\
+# \"dnssec-validation auto;\" is set and\n\
 #  sysconfdir/bind.keys doesn't exist).\n\
 #\n\
 # BEGIN MANAGED KEYS\n"
@@ -281,17 +311,19 @@ MANAGED_KEYS
 ";
 
 isc_result_t
-ns_config_parsedefaults(cfg_parser_t *parser, cfg_obj_t **conf) {
+named_config_parsedefaults(cfg_parser_t *parser, cfg_obj_t **conf) {
 	isc_buffer_t b;
 
 	isc_buffer_init(&b, defaultconf, sizeof(defaultconf) - 1);
 	isc_buffer_add(&b, sizeof(defaultconf) - 1);
-	return (cfg_parse_buffer(parser, &b, &cfg_type_namedconf, conf));
+	return (cfg_parse_buffer4(parser, &b, __FILE__, 0,
+				  &cfg_type_namedconf,
+				  CFG_PCTX_NODEPRECATED, conf));
 }
 
 isc_result_t
-ns_config_get(cfg_obj_t const * const *maps, const char *name,
-	      const cfg_obj_t **obj)
+named_config_get(cfg_obj_t const * const *maps, const char *name,
+		 const cfg_obj_t **obj)
 {
 	int i;
 
@@ -304,8 +336,8 @@ ns_config_get(cfg_obj_t const * const *maps, const char *name,
 }
 
 isc_result_t
-ns_checknames_get(const cfg_obj_t **maps, const char *which,
-		  const cfg_obj_t **obj)
+named_checknames_get(const cfg_obj_t **maps, const char *which,
+		     const cfg_obj_t **obj)
 {
 	const cfg_listelt_t *element;
 	const cfg_obj_t *checknames;
@@ -343,7 +375,7 @@ ns_checknames_get(const cfg_obj_t **maps, const char *which,
 }
 
 int
-ns_config_listcount(const cfg_obj_t *list) {
+named_config_listcount(const cfg_obj_t *list) {
 	const cfg_listelt_t *e;
 	int i = 0;
 
@@ -354,8 +386,9 @@ ns_config_listcount(const cfg_obj_t *list) {
 }
 
 isc_result_t
-ns_config_getclass(const cfg_obj_t *classobj, dns_rdataclass_t defclass,
-		   dns_rdataclass_t *classp) {
+named_config_getclass(const cfg_obj_t *classobj, dns_rdataclass_t defclass,
+		      dns_rdataclass_t *classp)
+{
 	isc_textregion_t r;
 	isc_result_t result;
 
@@ -367,14 +400,15 @@ ns_config_getclass(const cfg_obj_t *classobj, dns_rdataclass_t defclass,
 	r.length = strlen(r.base);
 	result = dns_rdataclass_fromtext(classp, &r);
 	if (result != ISC_R_SUCCESS)
-		cfg_obj_log(classobj, ns_g_lctx, ISC_LOG_ERROR,
+		cfg_obj_log(classobj, named_g_lctx, ISC_LOG_ERROR,
 			    "unknown class '%s'", r.base);
 	return (result);
 }
 
 isc_result_t
-ns_config_gettype(const cfg_obj_t *typeobj, dns_rdatatype_t deftype,
-		   dns_rdatatype_t *typep) {
+named_config_gettype(const cfg_obj_t *typeobj, dns_rdatatype_t deftype,
+		     dns_rdatatype_t *typep)
+{
 	isc_textregion_t r;
 	isc_result_t result;
 
@@ -386,37 +420,39 @@ ns_config_gettype(const cfg_obj_t *typeobj, dns_rdatatype_t deftype,
 	r.length = strlen(r.base);
 	result = dns_rdatatype_fromtext(typep, &r);
 	if (result != ISC_R_SUCCESS)
-		cfg_obj_log(typeobj, ns_g_lctx, ISC_LOG_ERROR,
+		cfg_obj_log(typeobj, named_g_lctx, ISC_LOG_ERROR,
 			    "unknown type '%s'", r.base);
 	return (result);
 }
 
 dns_zonetype_t
-ns_config_getzonetype(const cfg_obj_t *zonetypeobj) {
+named_config_getzonetype(const cfg_obj_t *zonetypeobj) {
 	dns_zonetype_t ztype = dns_zone_none;
 	const char *str;
 
 	str = cfg_obj_asstring(zonetypeobj);
-	if (strcasecmp(str, "master") == 0)
+	if (strcasecmp(str, "master") == 0) {
 		ztype = dns_zone_master;
-	else if (strcasecmp(str, "slave") == 0)
+	} else if (strcasecmp(str, "slave") == 0) {
 		ztype = dns_zone_slave;
-	else if (strcasecmp(str, "stub") == 0)
+	} else if (strcasecmp(str, "stub") == 0) {
 		ztype = dns_zone_stub;
-	else if (strcasecmp(str, "static-stub") == 0)
+	} else if (strcasecmp(str, "static-stub") == 0) {
 		ztype = dns_zone_staticstub;
-	else if (strcasecmp(str, "redirect") == 0)
+	} else if (strcasecmp(str, "redirect") == 0) {
 		ztype = dns_zone_redirect;
-	else
+	} else {
 		INSIST(0);
+		ISC_UNREACHABLE();
+	}
 	return (ztype);
 }
 
 isc_result_t
-ns_config_getiplist(const cfg_obj_t *config, const cfg_obj_t *list,
+named_config_getiplist(const cfg_obj_t *config, const cfg_obj_t *list,
 		    in_port_t defport, isc_mem_t *mctx,
 		    isc_sockaddr_t **addrsp, isc_dscp_t **dscpsp,
-		    isc_uint32_t *countp)
+		    uint32_t *countp)
 {
 	int count, i = 0;
 	const cfg_obj_t *addrlist;
@@ -432,13 +468,13 @@ ns_config_getiplist(const cfg_obj_t *config, const cfg_obj_t *list,
 	INSIST(countp != NULL);
 
 	addrlist = cfg_tuple_get(list, "addresses");
-	count = ns_config_listcount(addrlist);
+	count = named_config_listcount(addrlist);
 
 	portobj = cfg_tuple_get(list, "port");
 	if (cfg_obj_isuint32(portobj)) {
-		isc_uint32_t val = cfg_obj_asuint32(portobj);
-		if (val > ISC_UINT16_MAX) {
-			cfg_obj_log(portobj, ns_g_lctx, ISC_LOG_ERROR,
+		uint32_t val = cfg_obj_asuint32(portobj);
+		if (val > UINT16_MAX) {
+			cfg_obj_log(portobj, named_g_lctx, ISC_LOG_ERROR,
 				    "port '%u' out of range", val);
 			return (ISC_R_RANGE);
 		}
@@ -446,31 +482,35 @@ ns_config_getiplist(const cfg_obj_t *config, const cfg_obj_t *list,
 	} else if (defport != 0)
 		port = defport;
 	else {
-		result = ns_config_getport(config, &port);
+		result = named_config_getport(config, &port);
 		if (result != ISC_R_SUCCESS)
 			return (result);
 	}
 
 	if (dscpsp != NULL) {
-		dscps = isc_mem_get(mctx, count * sizeof(isc_dscp_t));
-		if (dscps == NULL)
-			return (ISC_R_NOMEMORY);
-
 		dscpobj = cfg_tuple_get(list, "dscp");
 		if (dscpobj != NULL && cfg_obj_isuint32(dscpobj)) {
 			if (cfg_obj_asuint32(dscpobj) > 63) {
-				cfg_obj_log(dscpobj, ns_g_lctx, ISC_LOG_ERROR,
+				cfg_obj_log(dscpobj, named_g_lctx,
+					    ISC_LOG_ERROR,
 					    "dscp value '%u' is out of range",
 					    cfg_obj_asuint32(dscpobj));
 				return (ISC_R_RANGE);
 			}
 			dscp = (isc_dscp_t)cfg_obj_asuint32(dscpobj);
 		}
+
+		dscps = isc_mem_get(mctx, count * sizeof(isc_dscp_t));
+		if (dscps == NULL)
+			return (ISC_R_NOMEMORY);
 	}
 
 	addrs = isc_mem_get(mctx, count * sizeof(isc_sockaddr_t));
-	if (addrs == NULL)
+	if (addrs == NULL) {
+		if (dscps != NULL)
+			isc_mem_put(mctx, dscps, count * sizeof(isc_dscp_t));
 		return (ISC_R_NOMEMORY);
+	}
 
 	for (element = cfg_list_first(addrlist);
 	     element != NULL;
@@ -502,8 +542,8 @@ ns_config_getiplist(const cfg_obj_t *config, const cfg_obj_t *list,
 }
 
 void
-ns_config_putiplist(isc_mem_t *mctx, isc_sockaddr_t **addrsp,
-		    isc_dscp_t **dscpsp, isc_uint32_t count)
+named_config_putiplist(isc_mem_t *mctx, isc_sockaddr_t **addrsp,
+		       isc_dscp_t **dscpsp, uint32_t count)
 {
 	INSIST(addrsp != NULL && *addrsp != NULL);
 	INSIST(dscpsp == NULL || *dscpsp != NULL);
@@ -546,21 +586,19 @@ get_masters_def(const cfg_obj_t *cctx, const char *name,
 }
 
 isc_result_t
-ns_config_getipandkeylist(const cfg_obj_t *config, const cfg_obj_t *list,
-			  isc_mem_t *mctx, isc_sockaddr_t **addrsp,
-			  isc_dscp_t **dscpsp, dns_name_t ***keysp,
-			  isc_uint32_t *countp)
+named_config_getipandkeylist(const cfg_obj_t *config, const cfg_obj_t *list,
+			  isc_mem_t *mctx, dns_ipkeylist_t *ipkl)
 {
-	isc_uint32_t addrcount = 0, dscpcount = 0, keycount = 0, i = 0;
-	isc_uint32_t listcount = 0, l = 0, j;
-	isc_uint32_t stackcount = 0, pushed = 0;
+	uint32_t addrcount = 0, dscpcount = 0, keycount = 0, i = 0;
+	uint32_t listcount = 0, l = 0, j;
+	uint32_t stackcount = 0, pushed = 0;
 	isc_result_t result;
 	const cfg_listelt_t *element;
 	const cfg_obj_t *addrlist;
 	const cfg_obj_t *portobj;
 	const cfg_obj_t *dscpobj;
 	in_port_t port;
-	isc_dscp_t dscp;
+	isc_dscp_t dscp = -1;
 	dns_fixedname_t fname;
 	isc_sockaddr_t *addrs = NULL;
 	isc_dscp_t *dscps = NULL;
@@ -572,10 +610,24 @@ ns_config_getipandkeylist(const cfg_obj_t *config, const cfg_obj_t *list,
 		isc_dscp_t dscp;
 	} *stack = NULL;
 
-	REQUIRE(addrsp != NULL && *addrsp == NULL);
-	REQUIRE(dscpsp != NULL && *dscpsp == NULL);
-	REQUIRE(keysp != NULL && *keysp == NULL);
-	REQUIRE(countp != NULL);
+	REQUIRE(ipkl != NULL);
+	REQUIRE(ipkl->count == 0);
+	REQUIRE(ipkl->addrs == NULL);
+	REQUIRE(ipkl->keys == NULL);
+	REQUIRE(ipkl->dscps == NULL);
+	REQUIRE(ipkl->labels == NULL);
+	REQUIRE(ipkl->allocated == 0);
+
+	/*
+	 * Get system defaults.
+	 */
+	result = named_config_getport(config, &port);
+	if (result != ISC_R_SUCCESS)
+		goto cleanup;
+
+	result = named_config_getdscp(config, &dscp);
+	if (result != ISC_R_SUCCESS)
+		goto cleanup;
 
  newlist:
 	addrlist = cfg_tuple_get(list, "addresses");
@@ -583,32 +635,25 @@ ns_config_getipandkeylist(const cfg_obj_t *config, const cfg_obj_t *list,
 	dscpobj = cfg_tuple_get(list, "dscp");
 
 	if (cfg_obj_isuint32(portobj)) {
-		isc_uint32_t val = cfg_obj_asuint32(portobj);
-		if (val > ISC_UINT16_MAX) {
-			cfg_obj_log(portobj, ns_g_lctx, ISC_LOG_ERROR,
+		uint32_t val = cfg_obj_asuint32(portobj);
+		if (val > UINT16_MAX) {
+			cfg_obj_log(portobj, named_g_lctx, ISC_LOG_ERROR,
 				    "port '%u' out of range", val);
 			result = ISC_R_RANGE;
 			goto cleanup;
 		}
 		port = (in_port_t) val;
-	} else {
-		result = ns_config_getport(config, &port);
-		if (result != ISC_R_SUCCESS)
-			goto cleanup;
 	}
 
 	if (dscpobj != NULL && cfg_obj_isuint32(dscpobj)) {
 		if (cfg_obj_asuint32(dscpobj) > 63) {
-			cfg_obj_log(dscpobj, ns_g_lctx, ISC_LOG_ERROR,
+			cfg_obj_log(dscpobj, named_g_lctx, ISC_LOG_ERROR,
 				    "dscp value '%u' is out of range",
 				    cfg_obj_asuint32(dscpobj));
-			return (ISC_R_RANGE);
+			result = ISC_R_RANGE;
+			goto cleanup;
 		}
 		dscp = (isc_dscp_t)cfg_obj_asuint32(dscpobj);
-	} else {
-		result = ns_config_getdscp(config, &dscp);
-		if (result != ISC_R_SUCCESS)
-			goto cleanup;
 	}
 
 	result = ISC_R_NOMEMORY;
@@ -634,20 +679,20 @@ ns_config_getipandkeylist(const cfg_obj_t *config, const cfg_obj_t *list,
 
 			/* Grow lists? */
 			if (listcount == l) {
-				void * new;
-				isc_uint32_t newlen = listcount + 16;
+				void * tmp;
+				uint32_t newlen = listcount + 16;
 				size_t newsize, oldsize;
 
 				newsize = newlen * sizeof(*lists);
 				oldsize = listcount * sizeof(*lists);
-				new = isc_mem_get(mctx, newsize);
-				if (new == NULL)
+				tmp = isc_mem_get(mctx, newsize);
+				if (tmp == NULL)
 					goto cleanup;
 				if (listcount != 0) {
-					memmove(new, lists, oldsize);
+					memmove(tmp, lists, oldsize);
 					isc_mem_put(mctx, lists, oldsize);
 				}
-				lists = new;
+				lists = tmp;
 				listcount = newlen;
 			}
 			/* Seen? */
@@ -658,7 +703,7 @@ ns_config_getipandkeylist(const cfg_obj_t *config, const cfg_obj_t *list,
 				continue;
 			tresult = get_masters_def(config, listname, &list);
 			if (tresult == ISC_R_NOTFOUND) {
-				cfg_obj_log(addr, ns_g_lctx, ISC_LOG_ERROR,
+				cfg_obj_log(addr, named_g_lctx, ISC_LOG_ERROR,
 				    "masters \"%s\" not found", listname);
 
 				result = tresult;
@@ -669,20 +714,20 @@ ns_config_getipandkeylist(const cfg_obj_t *config, const cfg_obj_t *list,
 			lists[l++].name = listname;
 			/* Grow stack? */
 			if (stackcount == pushed) {
-				void * new;
-				isc_uint32_t newlen = stackcount + 16;
+				void * tmp;
+				uint32_t newlen = stackcount + 16;
 				size_t newsize, oldsize;
 
 				newsize = newlen * sizeof(*stack);
 				oldsize = stackcount * sizeof(*stack);
-				new = isc_mem_get(mctx, newsize);
-				if (new == NULL)
+				tmp = isc_mem_get(mctx, newsize);
+				if (tmp == NULL)
 					goto cleanup;
 				if (stackcount != 0) {
-					memmove(new, stack, oldsize);
+					memmove(tmp, stack, oldsize);
 					isc_mem_put(mctx, stack, oldsize);
 				}
-				stack = new;
+				stack = tmp;
 				stackcount = newlen;
 			}
 			/*
@@ -697,44 +742,44 @@ ns_config_getipandkeylist(const cfg_obj_t *config, const cfg_obj_t *list,
 		}
 
 		if (i == addrcount) {
-			void * new;
-			isc_uint32_t newlen = addrcount + 16;
+			void * tmp;
+			uint32_t newlen = addrcount + 16;
 			size_t newsize, oldsize;
 
 			newsize = newlen * sizeof(isc_sockaddr_t);
 			oldsize = addrcount * sizeof(isc_sockaddr_t);
-			new = isc_mem_get(mctx, newsize);
-			if (new == NULL)
+			tmp = isc_mem_get(mctx, newsize);
+			if (tmp == NULL)
 				goto cleanup;
 			if (addrcount != 0) {
-				memmove(new, addrs, oldsize);
+				memmove(tmp, addrs, oldsize);
 				isc_mem_put(mctx, addrs, oldsize);
 			}
-			addrs = new;
+			addrs = tmp;
 			addrcount = newlen;
 
 			newsize = newlen * sizeof(isc_dscp_t);
 			oldsize = dscpcount * sizeof(isc_dscp_t);
-			new = isc_mem_get(mctx, newsize);
-			if (new == NULL)
+			tmp = isc_mem_get(mctx, newsize);
+			if (tmp == NULL)
 				goto cleanup;
 			if (dscpcount != 0) {
-				memmove(new, dscps, oldsize);
+				memmove(tmp, dscps, oldsize);
 				isc_mem_put(mctx, dscps, oldsize);
 			}
-			dscps = new;
+			dscps = tmp;
 			dscpcount = newlen;
 
 			newsize = newlen * sizeof(dns_name_t *);
 			oldsize = keycount * sizeof(dns_name_t *);
-			new = isc_mem_get(mctx, newsize);
-			if (new == NULL)
+			tmp = isc_mem_get(mctx, newsize);
+			if (tmp == NULL)
 				goto cleanup;
 			if (keycount != 0) {
-				memmove(new, keys, oldsize);
+				memmove(tmp, keys, oldsize);
 				isc_mem_put(mctx, keys, oldsize);
 			}
-			keys = new;
+			keys = tmp;
 			keycount = newlen;
 		}
 
@@ -774,46 +819,46 @@ ns_config_getipandkeylist(const cfg_obj_t *config, const cfg_obj_t *list,
 		goto resume;
 	}
 	if (i < addrcount) {
-		void * new;
+		void * tmp;
 		size_t newsize, oldsize;
 
 		newsize = i * sizeof(isc_sockaddr_t);
 		oldsize = addrcount * sizeof(isc_sockaddr_t);
 		if (i != 0) {
-			new = isc_mem_get(mctx, newsize);
-			if (new == NULL)
+			tmp = isc_mem_get(mctx, newsize);
+			if (tmp == NULL)
 				goto cleanup;
-			memmove(new, addrs, newsize);
+			memmove(tmp, addrs, newsize);
 		} else
-			new = NULL;
+			tmp = NULL;
 		isc_mem_put(mctx, addrs, oldsize);
-		addrs = new;
+		addrs = tmp;
 		addrcount = i;
 
 		newsize = i * sizeof(isc_dscp_t);
 		oldsize = dscpcount * sizeof(isc_dscp_t);
 		if (i != 0) {
-			new = isc_mem_get(mctx, newsize);
-			if (new == NULL)
+			tmp = isc_mem_get(mctx, newsize);
+			if (tmp == NULL)
 				goto cleanup;
-			memmove(new, dscps, newsize);
+			memmove(tmp, dscps, newsize);
 		} else
-			new = NULL;
+			tmp = NULL;
 		isc_mem_put(mctx, dscps, oldsize);
-		dscps = new;
+		dscps = tmp;
 		dscpcount = i;
 
 		newsize = i * sizeof(dns_name_t *);
 		oldsize = keycount * sizeof(dns_name_t *);
 		if (i != 0) {
-			new = isc_mem_get(mctx, newsize);
-			if (new == NULL)
+			tmp = isc_mem_get(mctx, newsize);
+			if (tmp == NULL)
 				goto cleanup;
-			memmove(new, keys,  newsize);
+			memmove(tmp, keys,  newsize);
 		} else
-			new = NULL;
+			tmp = NULL;
 		isc_mem_put(mctx, keys, oldsize);
-		keys = new;
+		keys = tmp;
 		keycount = i;
 	}
 
@@ -824,10 +869,11 @@ ns_config_getipandkeylist(const cfg_obj_t *config, const cfg_obj_t *list,
 
 	INSIST(keycount == addrcount);
 
-	*addrsp = addrs;
-	*dscpsp = dscps;
-	*keysp = keys;
-	*countp = addrcount;
+	ipkl->addrs = addrs;
+	ipkl->dscps = dscps;
+	ipkl->keys = keys;
+	ipkl->count = addrcount;
+	ipkl->allocated = addrcount;
 
 	return (ISC_R_SUCCESS);
 
@@ -853,39 +899,8 @@ ns_config_getipandkeylist(const cfg_obj_t *config, const cfg_obj_t *list,
 	return (result);
 }
 
-void
-ns_config_putipandkeylist(isc_mem_t *mctx, isc_sockaddr_t **addrsp,
-			  isc_dscp_t **dscpsp, dns_name_t ***keysp,
-			  isc_uint32_t count)
-{
-	unsigned int i;
-	dns_name_t **keys;
-
-	REQUIRE(addrsp != NULL && *addrsp != NULL);
-	REQUIRE(dscpsp == NULL || *dscpsp != NULL);
-	REQUIRE(keysp != NULL && *keysp != NULL);
-
-	keys = *keysp;
-
-	isc_mem_put(mctx, *addrsp, count * sizeof(isc_sockaddr_t));
-	if (dscpsp != NULL)
-		isc_mem_put(mctx, *dscpsp, count * sizeof(isc_dscp_t));
-	for (i = 0; i < count; i++) {
-		if (keys[i] == NULL)
-			continue;
-		if (dns_name_dynamic(keys[i]))
-			dns_name_free(keys[i], mctx);
-		isc_mem_put(mctx, keys[i], sizeof(dns_name_t));
-	}
-	isc_mem_put(mctx, *keysp, count * sizeof(dns_name_t *));
-	*addrsp = NULL;
-	if (dscpsp != NULL)
-		*dscpsp = NULL;
-	*keysp = NULL;
-}
-
 isc_result_t
-ns_config_getport(const cfg_obj_t *config, in_port_t *portp) {
+named_config_getport(const cfg_obj_t *config, in_port_t *portp) {
 	const cfg_obj_t *maps[3];
 	const cfg_obj_t *options = NULL;
 	const cfg_obj_t *portobj = NULL;
@@ -896,13 +911,13 @@ ns_config_getport(const cfg_obj_t *config, in_port_t *portp) {
 	i = 0;
 	if (options != NULL)
 		maps[i++] = options;
-	maps[i++] = ns_g_defaults;
+	maps[i++] = named_g_defaults;
 	maps[i] = NULL;
 
-	result = ns_config_get(maps, "port", &portobj);
+	result = named_config_get(maps, "port", &portobj);
 	INSIST(result == ISC_R_SUCCESS);
-	if (cfg_obj_asuint32(portobj) >= ISC_UINT16_MAX) {
-		cfg_obj_log(portobj, ns_g_lctx, ISC_LOG_ERROR,
+	if (cfg_obj_asuint32(portobj) >= UINT16_MAX) {
+		cfg_obj_log(portobj, named_g_lctx, ISC_LOG_ERROR,
 			    "port '%u' out of range",
 			    cfg_obj_asuint32(portobj));
 		return (ISC_R_RANGE);
@@ -912,7 +927,7 @@ ns_config_getport(const cfg_obj_t *config, in_port_t *portp) {
 }
 
 isc_result_t
-ns_config_getdscp(const cfg_obj_t *config, isc_dscp_t *dscpp) {
+named_config_getdscp(const cfg_obj_t *config, isc_dscp_t *dscpp) {
 	const cfg_obj_t *options = NULL;
 	const cfg_obj_t *dscpobj = NULL;
 	isc_result_t result;
@@ -927,7 +942,7 @@ ns_config_getdscp(const cfg_obj_t *config, isc_dscp_t *dscpp) {
 		return (ISC_R_SUCCESS);
 	}
 	if (cfg_obj_asuint32(dscpobj) >= 64) {
-		cfg_obj_log(dscpobj, ns_g_lctx, ISC_LOG_ERROR,
+		cfg_obj_log(dscpobj, named_g_lctx, ISC_LOG_ERROR,
 			    "dscp '%u' out of range",
 			    cfg_obj_asuint32(dscpobj));
 		return (ISC_R_RANGE);
@@ -941,11 +956,13 @@ struct keyalgorithms {
 	enum { hmacnone, hmacmd5, hmacsha1, hmacsha224,
 	       hmacsha256, hmacsha384, hmacsha512 } hmac;
 	unsigned int type;
-	isc_uint16_t size;
+	uint16_t size;
 } algorithms[] = {
+#ifndef PK11_MD5_DISABLE
 	{ "hmac-md5", hmacmd5, DST_ALG_HMACMD5, 128 },
 	{ "hmac-md5.sig-alg.reg.int", hmacmd5, DST_ALG_HMACMD5, 0 },
 	{ "hmac-md5.sig-alg.reg.int.", hmacmd5, DST_ALG_HMACMD5, 0 },
+#endif
 	{ "hmac-sha1", hmacsha1, DST_ALG_HMACSHA1, 160 },
 	{ "hmac-sha224", hmacsha224, DST_ALG_HMACSHA224, 224 },
 	{ "hmac-sha256", hmacsha256, DST_ALG_HMACSHA256, 256 },
@@ -955,19 +972,19 @@ struct keyalgorithms {
 };
 
 isc_result_t
-ns_config_getkeyalgorithm(const char *str, dns_name_t **name,
-			  isc_uint16_t *digestbits)
+named_config_getkeyalgorithm(const char *str, const dns_name_t **name,
+			     uint16_t *digestbits)
 {
-	return (ns_config_getkeyalgorithm2(str, name, NULL, digestbits));
+	return (named_config_getkeyalgorithm2(str, name, NULL, digestbits));
 }
 
 isc_result_t
-ns_config_getkeyalgorithm2(const char *str, dns_name_t **name,
-			   unsigned int *typep, isc_uint16_t *digestbits)
+named_config_getkeyalgorithm2(const char *str, const dns_name_t **name,
+			      unsigned int *typep, uint16_t *digestbits)
 {
 	int i;
 	size_t len = 0;
-	isc_uint16_t bits;
+	uint16_t bits;
 	isc_result_t result;
 
 	for (i = 0; algorithms[i].str != NULL; i++) {
@@ -992,7 +1009,9 @@ ns_config_getkeyalgorithm2(const char *str, dns_name_t **name,
 
 	if (name != NULL) {
 		switch (algorithms[i].hmac) {
+#ifndef PK11_MD5_DISABLE
 		case hmacmd5: *name = dns_tsig_hmacmd5_name; break;
+#endif
 		case hmacsha1: *name = dns_tsig_hmacsha1_name; break;
 		case hmacsha224: *name = dns_tsig_hmacsha224_name; break;
 		case hmacsha256: *name = dns_tsig_hmacsha256_name; break;
@@ -1000,6 +1019,7 @@ ns_config_getkeyalgorithm2(const char *str, dns_name_t **name,
 		case hmacsha512: *name = dns_tsig_hmacsha512_name; break;
 		default:
 			INSIST(0);
+			ISC_UNREACHABLE();
 		}
 	}
 	if (typep != NULL)
