@@ -17,7 +17,7 @@ test -r $RANDFILE || $GENRANDOM $RANDOMSIZE $RANDFILE
 prog=$0
 
 args="-r $RANDFILE"
-alg="-a RSAMD5 -b 512"
+alg="-a RSAMD5 -b 1024"
 quiet=0
 
 msg1="cryptography"
@@ -29,7 +29,7 @@ while test "$#" -gt 0; do
                 quiet=1
                 ;;
         rsa|RSA)
-                alg=""
+                alg="-a RSASHA1"
                 msg1="RSA cryptography"
                 ;;
         gost|GOST)
