@@ -55,16 +55,20 @@
 
 #include <isc/buffer.h>
 #include <isc/file.h>
+#include <isc/log.h>
 #include <isc/mem.h>
+#include <isc/mutex.h>
 #include <isc/once.h>
 #include <isc/print.h>
 #include <isc/sockaddr.h>
+#include <isc/task.h>
 #include <isc/thread.h>
 #include <isc/time.h>
 #include <isc/types.h>
 #include <isc/util.h>
 
 #include <dns/dnstap.h>
+#include <dns/events.h>
 #include <dns/log.h>
 #include <dns/message.h>
 #include <dns/name.h>
@@ -105,10 +109,17 @@ struct dns_dtenv {
 	struct fstrm_iothr *iothr;
 	struct fstrm_iothr_options *fopt;
 
+	isc_task_t *reopen_task;
+	isc_mutex_t reopen_lock;	/* locks 'reopen_queued' */
+	bool reopen_queued;
+
 	isc_region_t identity;
 	isc_region_t version;
 	char *path;
 	dns_dtmode_t mode;
+	isc_offset_t max_size;
+	int rolls;
+	isc_log_rollsuffix_t suffix;
 	isc_stats_t *stats;
 };
 
@@ -177,6 +188,14 @@ unlock:
 isc_result_t
 dns_dt_create(isc_mem_t *mctx, dns_dtmode_t mode, const char *path,
 	      struct fstrm_iothr_options **foptp, dns_dtenv_t **envp)
+{
+	return (dns_dt_create2(mctx, mode, path, foptp, NULL, envp));
+}
+
+isc_result_t
+dns_dt_create2(isc_mem_t *mctx, dns_dtmode_t mode, const char *path,
+	       struct fstrm_iothr_options **foptp, isc_task_t *reopen_task,
+	       dns_dtenv_t **envp)
 {
 	isc_result_t result = ISC_R_SUCCESS;
 	fstrm_res res;
@@ -188,7 +207,7 @@ dns_dt_create(isc_mem_t *mctx, dns_dtmode_t mode, const char *path,
 
 	REQUIRE(path != NULL);
 	REQUIRE(envp != NULL && *envp == NULL);
-	REQUIRE(foptp!= NULL && *foptp != NULL);
+	REQUIRE(foptp != NULL && *foptp != NULL);
 
 	isc_log_write(dns_lctx, DNS_LOGCATEGORY_DNSTAP,
 		      DNS_LOGMODULE_DNSTAP, ISC_LOG_INFO,
@@ -197,26 +216,27 @@ dns_dt_create(isc_mem_t *mctx, dns_dtmode_t mode, const char *path,
 	generation++;
 
 	env = isc_mem_get(mctx, sizeof(dns_dtenv_t));
-	if (env == NULL)
-		CHECK(ISC_R_NOMEMORY);
 
 	memset(env, 0, sizeof(dns_dtenv_t));
+	env->reopen_task = reopen_task;
+	isc_mutex_init(&env->reopen_lock);
+	env->reopen_queued = false;
+	env->path = isc_mem_strdup(mctx, path);
 
 	CHECK(isc_refcount_init(&env->refcount, 1));
 	CHECK(isc_stats_create(mctx, &env->stats, dns_dnstapcounter_max));
-	env->path = isc_mem_strdup(mctx, path);
-	if (env->path == NULL)
-		CHECK(ISC_R_NOMEMORY);
 
 	fwopt = fstrm_writer_options_init();
-	if (fwopt == NULL)
+	if (fwopt == NULL) {
 		CHECK(ISC_R_NOMEMORY);
+	}
 
 	res = fstrm_writer_options_add_content_type(fwopt,
 					    DNSTAP_CONTENT_TYPE,
 					    sizeof(DNSTAP_CONTENT_TYPE) - 1);
-	if (res != fstrm_res_success)
+	if (res != fstrm_res_success) {
 		CHECK(ISC_R_FAILURE);
+	}
 
 	if (mode == dns_dtmode_file) {
 		ffwopt = fstrm_file_options_init();
@@ -231,11 +251,13 @@ dns_dt_create(isc_mem_t *mctx, dns_dtmode_t mode, const char *path,
 								  env->path);
 			fw = fstrm_unix_writer_init(fuwopt, fwopt);
 		}
-	} else
+	} else {
 		CHECK(ISC_R_FAILURE);
+	}
 
-	if (fw == NULL)
+	if (fw == NULL) {
 		CHECK(ISC_R_FAILURE);
+	}
 
 	env->iothr = fstrm_iothr_init(*foptp, &fw);
 	if (env->iothr == NULL) {
@@ -246,6 +268,8 @@ dns_dt_create(isc_mem_t *mctx, dns_dtmode_t mode, const char *path,
 		CHECK(ISC_R_FAILURE);
 	}
 	env->mode = mode;
+	env->max_size = 0;
+	env->rolls = ISC_LOG_ROLLINFINITE;
 	env->fopt = *foptp;
 	*foptp = NULL;
 
@@ -265,20 +289,44 @@ dns_dt_create(isc_mem_t *mctx, dns_dtmode_t mode, const char *path,
 		fstrm_writer_options_destroy(&fwopt);
 
 	if (result != ISC_R_SUCCESS) {
-		if (env != NULL) {
-			if (env->mctx != NULL)
-				isc_mem_detach(&env->mctx);
-			if (env->path != NULL)
-				isc_mem_free(mctx, env->path);
-			if (env->stats != NULL)
-				isc_stats_detach(&env->stats);
-			isc_mem_put(mctx, env, sizeof(dns_dtenv_t));
+		isc_mutex_destroy(&env->reopen_lock);
+		isc_mem_free(mctx, env->path);
+		if (env->stats != NULL) {
+			isc_stats_detach(&env->stats);
 		}
+		isc_mem_putanddetach(&env->mctx, env, sizeof(dns_dtenv_t));
 	}
 
 	return (result);
 }
 
+isc_result_t
+dns_dt_setupfile(dns_dtenv_t *env, uint64_t max_size, int rolls,
+		 isc_log_rollsuffix_t suffix)
+{
+	REQUIRE(VALID_DTENV(env));
+
+	/*
+	 * If we're using unix domain socket mode, then any
+	 * change from the default values is invalid.
+	 */
+	if (env->mode == dns_dtmode_unix) {
+		if (max_size == 0 && rolls == ISC_LOG_ROLLINFINITE &&
+		    suffix == isc_log_rollsuffix_increment)
+		{
+			return (ISC_R_SUCCESS);
+		} else {
+			return (ISC_R_INVALIDFILE);
+		}
+	}
+
+	env->max_size = max_size;
+	env->rolls = rolls;
+	env->suffix = suffix;
+
+	return (ISC_R_SUCCESS);
+}
+
 isc_result_t
 dns_dt_reopen(dns_dtenv_t *env, int roll) {
 	isc_result_t result = ISC_R_SUCCESS;
@@ -291,18 +339,27 @@ dns_dt_reopen(dns_dtenv_t *env, int roll) {
 
 	REQUIRE(VALID_DTENV(env));
 
+	/*
+	 * Run in task-exclusive mode.
+	 */
+	result = isc_task_beginexclusive(env->reopen_task);
+	RUNTIME_CHECK(result == ISC_R_SUCCESS);
+
 	/*
 	 * Check that we can create a new fw object.
 	 */
 	fwopt = fstrm_writer_options_init();
-	if (fwopt == NULL)
+	if (fwopt == NULL) {
+		isc_task_endexclusive(env->reopen_task);
 		return (ISC_R_NOMEMORY);
+	}
 
 	res = fstrm_writer_options_add_content_type(fwopt,
 					    DNSTAP_CONTENT_TYPE,
 					    sizeof(DNSTAP_CONTENT_TYPE) - 1);
-	if (res != fstrm_res_success)
+	if (res != fstrm_res_success) {
 		CHECK(ISC_R_FAILURE);
+	}
 
 	if (env->mode == dns_dtmode_file) {
 		ffwopt = fstrm_file_options_init();
@@ -317,11 +374,13 @@ dns_dt_reopen(dns_dtenv_t *env, int roll) {
 								  env->path);
 			fw = fstrm_unix_writer_init(fuwopt, fwopt);
 		}
-	} else
+	} else {
 		CHECK(ISC_R_NOTIMPLEMENTED);
+	}
 
-	if (fw == NULL)
+	if (fw == NULL) {
 		CHECK(ISC_R_FAILURE);
+	}
 
 	/*
 	 * We are committed here.
@@ -334,10 +393,15 @@ dns_dt_reopen(dns_dtenv_t *env, int roll) {
 
 	generation++;
 
-	if (env->iothr != NULL)
+	if (env->iothr != NULL) {
 		fstrm_iothr_destroy(&env->iothr);
+	}
 
-	if (env->mode == dns_dtmode_file && roll >= 0) {
+	if (roll != 0) {
+		roll = env->rolls;
+	}
+
+	if (env->mode == dns_dtmode_file && roll != 0) {
 		/*
 		 * Create a temporary isc_logfile_t structure so we can
 		 * take advantage of the logfile rolling facility.
@@ -345,9 +409,10 @@ dns_dt_reopen(dns_dtenv_t *env, int roll) {
 		char *filename = isc_mem_strdup(env->mctx, env->path);
 		file.name = filename;
 		file.stream = NULL;
-		file.versions = roll != 0 ? roll : ISC_LOG_ROLLINFINITE;
+		file.versions = roll != 0 ? roll : env->rolls;
 		file.maximum_size = 0;
 		file.maximum_reached = false;
+		file.suffix = env->suffix;
 		result = isc_logfile_roll(&file);
 		isc_mem_free(env->mctx, filename);
 		CHECK(result);
@@ -374,6 +439,8 @@ dns_dt_reopen(dns_dtenv_t *env, int roll) {
 	if (fuwopt != NULL)
 		fstrm_unix_writer_options_destroy(&fuwopt);
 
+	isc_task_endexclusive(env->reopen_task);
+
 	return (result);
 }
 
@@ -678,6 +745,91 @@ setaddr(dns_dtmsg_t *dm, isc_sockaddr_t *sa, bool tcp,
 	*has_port = 1;
 }
 
+/*%
+ * Invoke dns_dt_reopen() and re-allow dnstap output file rolling.  This
+ * function is run in the context of the task stored in the 'reopen_task' field
+ * of the dnstap environment structure.
+ */
+static void
+perform_reopen(isc_task_t *task, isc_event_t *event) {
+	dns_dtenv_t *env;
+
+	REQUIRE(event != NULL);
+	REQUIRE(event->ev_type == DNS_EVENT_FREESTORAGE);
+
+	env = (dns_dtenv_t *)event->ev_arg;
+
+	REQUIRE(VALID_DTENV(env));
+	REQUIRE(task == env->reopen_task);
+
+	/*
+	 * Roll output file in the context of env->reopen_task.
+	 */
+	dns_dt_reopen(env, env->rolls);
+
+	/*
+	 * Clean up.
+	 */
+	isc_event_free(&event);
+	isc_task_detach(&task);
+
+	/*
+	 * Re-allow output file rolling.
+	 */
+	LOCK(&env->reopen_lock);
+	env->reopen_queued = false;
+	UNLOCK(&env->reopen_lock);
+}
+
+/*%
+ * Check whether a dnstap output file roll is due and if so, initiate it (the
+ * actual roll happens asynchronously).
+ */
+static void
+check_file_size_and_maybe_reopen(dns_dtenv_t *env) {
+	isc_task_t *reopen_task = NULL;
+	isc_event_t *event;
+	struct stat statbuf;
+
+	/*
+	 * If the task from which the output file should be reopened was not
+	 * specified, abort.
+	 */
+	if (env->reopen_task == NULL) {
+		return;
+	}
+
+	/*
+	 * If an output file roll is not currently queued, check the current
+	 * size of the output file to see whether a roll is needed.  Return if
+	 * it is not.
+	 */
+	LOCK(&env->reopen_lock);
+	if (env->reopen_queued || stat(env->path, &statbuf) < 0 ||
+	    statbuf.st_size <= env->max_size)
+	{
+		goto unlock_and_return;
+	}
+
+	/*
+	 * We need to roll the output file, but it needs to be done in the
+	 * context of env->reopen_task.  Allocate and send an event to achieve
+	 * that, then disallow output file rolling until the roll we queue is
+	 * completed.
+	 */
+	event = isc_event_allocate(env->mctx, NULL, DNS_EVENT_FREESTORAGE,
+				   perform_reopen, env, sizeof(*event));
+	if (event == NULL) {
+		goto unlock_and_return;
+	}
+	isc_task_attach(env->reopen_task, &reopen_task);
+	isc_task_send(reopen_task, &event);
+	env->reopen_queued = true;
+
+ unlock_and_return:
+	UNLOCK(&env->reopen_lock);
+}
+
 void
 dns_dt_send(dns_view_t *view, dns_dtmsgtype_t msgtype,
 	    isc_sockaddr_t *qaddr, isc_sockaddr_t *raddr,
@@ -697,6 +849,10 @@ dns_dt_send(dns_view_t *view, dns_dtmsgtype_t msgtype,
 
 	REQUIRE(VALID_DTENV(view->dtenv));
 
+	if (view->dtenv->max_size != 0) {
+		check_file_size_and_maybe_reopen(view->dtenv);
+	}
+
 	TIME_NOW(&now);
 	t = &now;
 
