@@ -1,30 +1,26 @@
 /*
- * Copyright (C) 2004, 2005, 2007-2011, 2013, 2014  Internet Systems Consortium, Inc. ("ISC")
- * Copyright (C) 1999-2002  Internet Software Consortium.
+ * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
- * Permission to use, copy, modify, and/or distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
- * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
- * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
- * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
- * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
- * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
+ * See the COPYRIGHT file distributed with this work for additional
+ * information regarding copyright ownership.
  */
 
-/* $Id: journal.c,v 1.120 2011/12/22 07:32:41 each Exp $ */
-
 #include <config.h>
 
+#include <inttypes.h>
+#include <stdbool.h>
+
 #include <stdlib.h>
 #include <unistd.h>
 #include <errno.h>
 
 #include <isc/file.h>
 #include <isc/mem.h>
+#include <isc/print.h>
 #include <isc/stdio.h>
 #include <isc/string.h>
 #include <isc/util.h>
@@ -83,7 +79,7 @@
  * SOA serial number does not change (BIND 8 sends such
  * sequences).
  */
-static isc_boolean_t bind8_compat = ISC_TRUE; /* XXX config */
+static bool bind8_compat = true; /* XXX config */
 
 /**************************************************************************/
 /*
@@ -115,7 +111,7 @@ static isc_boolean_t bind8_compat = ISC_TRUE; /* XXX config */
 
 static isc_result_t index_to_disk(dns_journal_t *);
 
-static inline isc_uint32_t
+static inline uint32_t
 decode_uint32(unsigned char *p) {
 	return ((p[0] << 24) +
 		(p[1] << 16) +
@@ -124,11 +120,11 @@ decode_uint32(unsigned char *p) {
 }
 
 static inline void
-encode_uint32(isc_uint32_t val, unsigned char *p) {
-	p[0] = (isc_uint8_t)(val >> 24);
-	p[1] = (isc_uint8_t)(val >> 16);
-	p[2] = (isc_uint8_t)(val >>  8);
-	p[3] = (isc_uint8_t)(val >>  0);
+encode_uint32(uint32_t val, unsigned char *p) {
+	p[0] = (uint8_t)(val >> 24);
+	p[1] = (uint8_t)(val >> 16);
+	p[2] = (uint8_t)(val >>  8);
+	p[3] = (uint8_t)(val >>  0);
 }
 
 isc_result_t
@@ -139,12 +135,14 @@ dns_db_createsoatuple(dns_db_t *db, dns_dbversion_t *ver, isc_mem_t *mctx,
 	dns_dbnode_t *node;
 	dns_rdataset_t rdataset;
 	dns_rdata_t rdata = DNS_RDATA_INIT;
+	dns_fixedname_t fixed;
 	dns_name_t *zonename;
 
-	zonename = dns_db_origin(db);
+	zonename = dns_fixedname_initname(&fixed);
+	dns_name_copy(dns_db_origin(db), zonename, NULL);
 
 	node = NULL;
-	result = dns_db_findnode(db, zonename, ISC_FALSE, &node);
+	result = dns_db_findnode(db, zonename, false, &node);
 	if (result != ISC_R_SUCCESS)
 		goto nonode;
 
@@ -159,6 +157,7 @@ dns_db_createsoatuple(dns_db_t *db, dns_dbversion_t *ver, isc_mem_t *mctx,
 		goto freenode;
 
 	dns_rdataset_current(&rdataset, &rdata);
+	dns_rdataset_getownercase(&rdataset, zonename);
 
 	result = dns_difftuple_create(mctx, op, zonename, rdataset.ttl,
 				      &rdata, tp);
@@ -245,7 +244,7 @@ typedef struct {
  * The in-core representation of the journal header.
  */
 typedef struct {
-	isc_uint32_t	serial;
+	uint32_t	serial;
 	isc_offset_t	offset;
 } journal_pos_t;
 
@@ -256,9 +255,9 @@ typedef struct {
 	unsigned char 	format[16];
 	journal_pos_t 	begin;
 	journal_pos_t 	end;
-	isc_uint32_t	index_size;
-	isc_uint32_t	sourceserial;
-	isc_boolean_t	serialset;
+	uint32_t	index_size;
+	uint32_t	sourceserial;
+	bool	serialset;
 } journal_header_t;
 
 /*%
@@ -266,16 +265,16 @@ typedef struct {
  */
 
 typedef struct {
-	isc_uint32_t	size;
-	isc_uint32_t	serial0;
-	isc_uint32_t	serial1;
+	uint32_t	size;
+	uint32_t	serial0;
+	uint32_t	serial1;
 } journal_xhdr_t;
 
 /*%
  * The in-core representation of the RR header.
  */
 typedef struct {
-	isc_uint32_t	size;
+	uint32_t	size;
 } journal_rrhdr_t;
 
 
@@ -326,13 +325,13 @@ struct dns_journal {
 		journal_pos_t bpos;		/*%< Position before first, */
 		journal_pos_t epos;		/*%< and after last transaction */
 		/* The rest is iterator state. */
-		isc_uint32_t current_serial;	/*%< Current SOA serial */
+		uint32_t current_serial;	/*%< Current SOA serial */
 		isc_buffer_t source;		/*%< Data from disk */
 		isc_buffer_t target;		/*%< Data from _fromwire check */
 		dns_decompress_t dctx;		/*%< Dummy decompression ctx */
 		dns_name_t name;		/*%< Current domain name */
 		dns_rdata_t rdata;		/*%< Current rdata */
-		isc_uint32_t ttl;		/*%< Current TTL */
+		uint32_t ttl;		/*%< Current TTL */
 		unsigned int xsize;		/*%< Size of transaction data */
 		unsigned int xpos;		/*%< Current position in it */
 		isc_result_t result;		/*%< Result of last call */
@@ -362,7 +361,7 @@ journal_header_decode(journal_rawheader_t *raw, journal_header_t *cooked) {
 	journal_pos_decode(&raw->h.end, &cooked->end);
 	cooked->index_size = decode_uint32(raw->h.index_size);
 	cooked->sourceserial = decode_uint32(raw->h.sourceserial);
-	cooked->serialset = ISC_TF(raw->h.flags & JOURNAL_SERIALSET);
+	cooked->serialset = ((raw->h.flags & JOURNAL_SERIALSET) != 0);
 }
 
 static void
@@ -376,8 +375,9 @@ journal_header_encode(journal_header_t *cooked, journal_rawheader_t *raw) {
 	journal_pos_encode(&raw->h.end, &cooked->end);
 	encode_uint32(cooked->index_size, raw->h.index_size);
 	encode_uint32(cooked->sourceserial, raw->h.sourceserial);
-	if (cooked->serialset)
+	if (cooked->serialset) {
 		flags |= JOURNAL_SERIALSET;
+	}
 	raw->h.flags = flags;
 }
 
@@ -385,7 +385,7 @@ journal_header_encode(journal_header_t *cooked, journal_rawheader_t *raw) {
  * Journal file I/O subroutines, with error checking and reporting.
  */
 static isc_result_t
-journal_seek(dns_journal_t *j, isc_uint32_t offset) {
+journal_seek(dns_journal_t *j, uint32_t offset) {
 	isc_result_t result;
 
 	result = isc_stdio_seek(j->fp, (off_t)offset, SEEK_SET);
@@ -469,8 +469,8 @@ journal_read_xhdr(dns_journal_t *j, journal_xhdr_t *xhdr) {
 }
 
 static isc_result_t
-journal_write_xhdr(dns_journal_t *j, isc_uint32_t size,
-		   isc_uint32_t serial0, isc_uint32_t serial1)
+journal_write_xhdr(dns_journal_t *j, uint32_t size,
+		   uint32_t serial0, uint32_t serial1)
 {
 	journal_rawxhdr_t raw;
 	encode_uint32(size, raw.size);
@@ -556,8 +556,8 @@ journal_file_create(isc_mem_t *mctx, const char *filename) {
 }
 
 static isc_result_t
-journal_open(isc_mem_t *mctx, const char *filename, isc_boolean_t write,
-	     isc_boolean_t create, dns_journal_t **journalp)
+journal_open(isc_mem_t *mctx, const char *filename, bool writable,
+	     bool create, dns_journal_t **journalp)
 {
 	FILE *fp = NULL;
 	isc_result_t result;
@@ -580,7 +580,7 @@ journal_open(isc_mem_t *mctx, const char *filename, isc_boolean_t write,
 	if (j->filename == NULL)
 		FAIL(ISC_R_NOMEMORY);
 
-	result = isc_stdio_open(j->filename, write ? "rb+" : "rb", &fp);
+	result = isc_stdio_open(j->filename, writable ? "rb+" : "rb", &fp);
 
 	if (result == ISC_R_FILENOTFOUND) {
 		if (create) {
@@ -670,18 +670,19 @@ journal_open(isc_mem_t *mctx, const char *filename, isc_boolean_t write,
 	dns_decompress_init(&j->it.dctx, -1, DNS_DECOMPRESS_NONE);
 
 	j->state =
-		write ? JOURNAL_STATE_WRITE : JOURNAL_STATE_READ;
+		writable ? JOURNAL_STATE_WRITE : JOURNAL_STATE_READ;
 
 	*journalp = j;
 	return (ISC_R_SUCCESS);
 
  failure:
 	j->magic = 0;
-	if (j->index != NULL) {
-		isc_mem_put(j->mctx, j->index, j->header.index_size *
+	if (j->rawindex != NULL)
+		isc_mem_put(j->mctx, j->rawindex, j->header.index_size *
 			    sizeof(journal_rawpos_t));
-		j->index = NULL;
-	}
+	if (j->index != NULL)
+		isc_mem_put(j->mctx, j->index, j->header.index_size *
+			    sizeof(journal_pos_t));
 	if (j->filename != NULL)
 		isc_mem_free(j->mctx, j->filename);
 	if (j->fp != NULL)
@@ -697,12 +698,12 @@ dns_journal_open(isc_mem_t *mctx, const char *filename, unsigned int mode,
 	isc_result_t result;
 	size_t namelen;
 	char backup[1024];
-	isc_boolean_t write, create;
+	bool writable, create;
 
-	create = ISC_TF(mode & DNS_JOURNAL_CREATE);
-	write = ISC_TF(mode & (DNS_JOURNAL_WRITE|DNS_JOURNAL_CREATE));
+	create = ((mode & DNS_JOURNAL_CREATE) != 0);
+	writable = ((mode & (DNS_JOURNAL_WRITE|DNS_JOURNAL_CREATE)) != 0);
 
-	result = journal_open(mctx, filename, write, create, journalp);
+	result = journal_open(mctx, filename, writable, create, journalp);
 	if (result == ISC_R_NOTFOUND) {
 		namelen = strlen(filename);
 		if (namelen > 4U && strcmp(filename + namelen - 4, ".jnl") == 0)
@@ -712,7 +713,8 @@ dns_journal_open(isc_mem_t *mctx, const char *filename, unsigned int mode,
 					   (int)namelen, filename);
 		if (result != ISC_R_SUCCESS)
 			return (result);
-		result = journal_open(mctx, backup, write, write, journalp);
+		result = journal_open(mctx, backup, writable, writable,
+				      journalp);
 	}
 	return (result);
 }
@@ -749,6 +751,7 @@ ixfr_order(const void *av, const void *bv) {
 		break;
 	default:
 		INSIST(0);
+		ISC_UNREACHABLE();
 	}
 
 	switch (b->op) {
@@ -762,6 +765,7 @@ ixfr_order(const void *av, const void *bv) {
 		break;
 	default:
 		INSIST(0);
+		ISC_UNREACHABLE();
 	}
 
 	r = bop - aop;
@@ -847,7 +851,7 @@ journal_next(dns_journal_t *j, journal_pos_t *pos) {
  * but not greater than 'serial'.
  */
 static void
-index_find(dns_journal_t *j, isc_uint32_t serial, journal_pos_t *best_guess) {
+index_find(dns_journal_t *j, uint32_t serial, journal_pos_t *best_guess) {
 	unsigned int i;
 	if (j->index == NULL)
 		return;
@@ -908,7 +912,7 @@ index_add(dns_journal_t *j, journal_pos_t *pos) {
  * ambiguous when a new transaction with number 'serial' is added.
  */
 static void
-index_invalidate(dns_journal_t *j, isc_uint32_t serial) {
+index_invalidate(dns_journal_t *j, uint32_t serial) {
 	unsigned int i;
 	if (j->index == NULL)
 		return;
@@ -938,7 +942,7 @@ index_invalidate(dns_journal_t *j, isc_uint32_t serial) {
  *
  */
 static isc_result_t
-journal_find(dns_journal_t *j, isc_uint32_t serial, journal_pos_t *pos) {
+journal_find(dns_journal_t *j, uint32_t serial, journal_pos_t *pos) {
 	isc_result_t result;
 	journal_pos_t current_pos;
 	REQUIRE(DNS_JOURNAL_VALID(j));
@@ -968,7 +972,7 @@ journal_find(dns_journal_t *j, isc_uint32_t serial, journal_pos_t *pos) {
 
 isc_result_t
 dns_journal_begin_transaction(dns_journal_t *j) {
-	isc_uint32_t offset;
+	uint32_t offset;
 	isc_result_t result;
 	journal_rawxhdr_t hdr;
 
@@ -1012,7 +1016,7 @@ dns_journal_writediff(dns_journal_t *j, dns_diff_t *diff) {
 	dns_difftuple_t *t;
 	isc_buffer_t buffer;
 	void *mem = NULL;
-	unsigned int size;
+	uint64_t size;
 	isc_result_t result;
 	isc_region_t used;
 
@@ -1042,6 +1046,14 @@ dns_journal_writediff(dns_journal_t *j, dns_diff_t *diff) {
 		size += t->rdata.length;
 	}
 
+	if (size >= DNS_JOURNAL_SIZE_MAX) {
+		isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
+			      "dns_journal_writediff: %s: journal entry "
+			      "too big to be stored: %" PRIu64 " bytes", j->filename,
+			      size);
+		return (ISC_R_NOSPACE);
+	}
+
 	mem = isc_mem_get(j->mctx, size);
 	if (mem == NULL)
 		return (ISC_R_NOMEMORY);
@@ -1067,7 +1079,7 @@ dns_journal_writediff(dns_journal_t *j, dns_diff_t *diff) {
 		isc_buffer_putuint16(&buffer, t->rdata.rdclass);
 		isc_buffer_putuint32(&buffer, t->ttl);
 		INSIST(t->rdata.length < 65536);
-		isc_buffer_putuint16(&buffer, (isc_uint16_t)t->rdata.length);
+		isc_buffer_putuint16(&buffer, (uint16_t)t->rdata.length);
 		INSIST(isc_buffer_availablelength(&buffer) >= t->rdata.length);
 		isc_buffer_putmem(&buffer, t->rdata.data, t->rdata.length);
 	}
@@ -1095,6 +1107,7 @@ isc_result_t
 dns_journal_commit(dns_journal_t *j) {
 	isc_result_t result;
 	journal_rawheader_t rawheader;
+	uint64_t total;
 
 	REQUIRE(DNS_JOURNAL_VALID(j));
 	REQUIRE(j->state == JOURNAL_STATE_TRANSACTION ||
@@ -1144,6 +1157,18 @@ dns_journal_commit(dns_journal_t *j) {
 		}
 	}
 
+	/*
+	 * We currently don't support huge journal entries.
+	 */
+	total = j->x.pos[1].offset - j->x.pos[0].offset;
+	if (total >= DNS_JOURNAL_SIZE_MAX) {
+		isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
+			     "transaction too big to be stored in journal: "
+			     "%" PRIu64 "b (max is %" PRIu64 "b)", total,
+			     (uint64_t)DNS_JOURNAL_SIZE_MAX);
+		return (ISC_R_UNEXPECTED);
+	}
+
 	/*
 	 * Some old journal entries may become non-addressable
 	 * when we increment the current serial number.  Purge them
@@ -1267,8 +1292,8 @@ static isc_result_t
 roll_forward(dns_journal_t *j, dns_db_t *db, unsigned int options) {
 	isc_buffer_t source;		/* Transaction data from disk */
 	isc_buffer_t target;		/* Ditto after _fromwire check */
-	isc_uint32_t db_serial;		/* Database SOA serial */
-	isc_uint32_t end_serial;	/* Last journal SOA serial */
+	uint32_t db_serial;		/* Database SOA serial */
+	uint32_t end_serial;	/* Last journal SOA serial */
 	isc_result_t result;
 	dns_dbversion_t *ver = NULL;
 	journal_pos_t pos;
@@ -1325,7 +1350,7 @@ roll_forward(dns_journal_t *j, dns_db_t *db, unsigned int options) {
 	     result = dns_journal_next_rr(j))
 	{
 		dns_name_t *name;
-		isc_uint32_t ttl;
+		uint32_t ttl;
 		dns_rdata_t *rdata;
 		dns_difftuple_t *tuple = NULL;
 
@@ -1383,7 +1408,7 @@ roll_forward(dns_journal_t *j, dns_db_t *db, unsigned int options) {
  failure:
 	if (ver != NULL)
 		dns_db_closeversion(db, &ver, result == ISC_R_SUCCESS ?
-				    ISC_TRUE : ISC_FALSE);
+				    true : false);
 
 	if (source.base != NULL)
 		isc_mem_put(j->mctx, source.base, source.length);
@@ -1431,8 +1456,8 @@ dns_journal_print(isc_mem_t *mctx, const char *filename, FILE *file) {
 	dns_journal_t *j;
 	isc_buffer_t source;		/* Transaction data from disk */
 	isc_buffer_t target;		/* Ditto after _fromwire check */
-	isc_uint32_t start_serial;		/* Database SOA serial */
-	isc_uint32_t end_serial;	/* Last journal SOA serial */
+	uint32_t start_serial;		/* Database SOA serial */
+	uint32_t end_serial;	/* Last journal SOA serial */
 	isc_result_t result;
 	dns_diff_t diff;
 	unsigned int n_soa = 0;
@@ -1476,7 +1501,7 @@ dns_journal_print(isc_mem_t *mctx, const char *filename, FILE *file) {
 	     result = dns_journal_next_rr(j))
 	{
 		dns_name_t *name;
-		isc_uint32_t ttl;
+		uint32_t ttl;
 		dns_rdata_t *rdata;
 		dns_difftuple_t *tuple = NULL;
 
@@ -1538,37 +1563,42 @@ dns_journal_print(isc_mem_t *mctx, const char *filename, FILE *file) {
 /*
  * Miscellaneous accessors.
  */
-isc_uint32_t
+bool
+dns_journal_empty(dns_journal_t *j) {
+	return (JOURNAL_EMPTY(&j->header));
+}
+
+uint32_t
 dns_journal_first_serial(dns_journal_t *j) {
 	return (j->header.begin.serial);
 }
 
-isc_uint32_t
+uint32_t
 dns_journal_last_serial(dns_journal_t *j) {
 	return (j->header.end.serial);
 }
 
 void
-dns_journal_set_sourceserial(dns_journal_t *j, isc_uint32_t sourceserial) {
+dns_journal_set_sourceserial(dns_journal_t *j, uint32_t sourceserial) {
 
 	REQUIRE(j->state == JOURNAL_STATE_WRITE ||
 		j->state == JOURNAL_STATE_INLINE ||
 		j->state == JOURNAL_STATE_TRANSACTION);
 
 	j->header.sourceserial = sourceserial;
-	j->header.serialset = ISC_TRUE;
+	j->header.serialset = true;
 	if (j->state == JOURNAL_STATE_WRITE)
 		j->state = JOURNAL_STATE_INLINE;
 }
 
-isc_boolean_t
-dns_journal_get_sourceserial(dns_journal_t *j, isc_uint32_t *sourceserial) {
+bool
+dns_journal_get_sourceserial(dns_journal_t *j, uint32_t *sourceserial) {
 	REQUIRE(sourceserial != NULL);
 
 	if (!j->header.serialset)
-		return (ISC_FALSE);
+		return (false);
 	*sourceserial = j->header.sourceserial;
-	return (ISC_TRUE);
+	return (true);
 }
 
 /**************************************************************************/
@@ -1611,7 +1641,7 @@ size_buffer(isc_mem_t *mctx, isc_buffer_t *b, unsigned size) {
 
 isc_result_t
 dns_journal_iter_init(dns_journal_t *j,
-		      isc_uint32_t begin_serial, isc_uint32_t end_serial)
+		      uint32_t begin_serial, uint32_t end_serial)
 {
 	isc_result_t result;
 
@@ -1655,11 +1685,16 @@ read_one_rr(dns_journal_t *j) {
 	dns_rdatatype_t rdtype;
 	dns_rdataclass_t rdclass;
 	unsigned int rdlen;
-	isc_uint32_t ttl;
+	uint32_t ttl;
 	journal_xhdr_t xhdr;
 	journal_rrhdr_t rrhdr;
 
-	INSIST(j->offset <= j->it.epos.offset);
+	if (j->offset > j->it.epos.offset) {
+		isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
+			      "%s: journal corrupt: possible integer overflow",
+			      j->filename);
+		return (ISC_R_UNEXPECTED);
+	}
 	if (j->offset == j->it.epos.offset)
 		return (ISC_R_NOMORE);
 	if (j->it.xpos == j->it.xsize) {
@@ -1768,7 +1803,7 @@ dns_journal_next_rr(dns_journal_t *j) {
 }
 
 void
-dns_journal_current_rr(dns_journal_t *j, dns_name_t **name, isc_uint32_t *ttl,
+dns_journal_current_rr(dns_journal_t *j, dns_name_t **name, uint32_t *ttl,
 		   dns_rdata_t **rdata)
 {
 	REQUIRE(j->it.result == ISC_R_SUCCESS);
@@ -1876,7 +1911,7 @@ dns_diff_subtract(dns_diff_t diff[2], dns_diff_t *r) {
 	isc_result_t result;
 	dns_difftuple_t *p[2];
 	int i, t;
-	isc_boolean_t append;
+	bool append;
 
 	CHECK(dns_diff_sort(&diff[0], rdata_order));
 	CHECK(dns_diff_sort(&diff[1], rdata_order));
@@ -1909,7 +1944,7 @@ dns_diff_subtract(dns_diff_t diff[2], dns_diff_t *r) {
 		 * Identical RRs in both databases; skip them both
 		 * if the ttl differs.
 		 */
-		append = ISC_TF(p[0]->ttl != p[1]->ttl);
+		append = (p[0]->ttl != p[1]->ttl);
 		for (i = 0; i < 2; i++) {
 			ISC_LIST_UNLINK(diff[i].tuples, p[i], link);
 			if (append) {
@@ -1933,7 +1968,7 @@ diff_namespace(dns_db_t *dba, dns_dbversion_t *dbvera,
 	dns_db_t *db[2];
 	dns_dbversion_t *ver[2];
 	dns_dbiterator_t *dbit[2] = { NULL, NULL };
-	isc_boolean_t have[2] = { ISC_FALSE, ISC_FALSE };
+	bool have[2] = { false, false };
 	dns_fixedname_t fixname[2];
 	isc_result_t result, itresult[2];
 	dns_diff_t diff[2];
@@ -1968,7 +2003,7 @@ diff_namespace(dns_db_t *dba, dns_dbversion_t *dbvera,
 					    DNS_DIFFOP_DEL,
 					    &diff[i]));
 				itresult[i] = dns_dbiterator_next(dbit[i]);
-				have[i] = ISC_TRUE;
+				have[i] = true;
 			}
 		}
 
@@ -1983,7 +2018,7 @@ diff_namespace(dns_db_t *dba, dns_dbversion_t *dbvera,
 				ISC_LIST_APPENDLIST(resultdiff->tuples,
 						    diff[i].tuples, link);
 				INSIST(ISC_LIST_EMPTY(diff[i].tuples));
-				have[i] = ISC_FALSE;
+				have[i] = false;
 				goto next;
 			}
 		}
@@ -1994,21 +2029,21 @@ diff_namespace(dns_db_t *dba, dns_dbversion_t *dbvera,
 			ISC_LIST_APPENDLIST(resultdiff->tuples,
 					    diff[0].tuples, link);
 			INSIST(ISC_LIST_EMPTY(diff[0].tuples));
-			have[0] = ISC_FALSE;
+			have[0] = false;
 			continue;
 		}
 		if (t > 0) {
 			ISC_LIST_APPENDLIST(resultdiff->tuples,
 					    diff[1].tuples, link);
 			INSIST(ISC_LIST_EMPTY(diff[1].tuples));
-			have[1] = ISC_FALSE;
+			have[1] = false;
 			continue;
 		}
 		INSIST(t == 0);
 		CHECK(dns_diff_subtract(diff, resultdiff));
 		INSIST(ISC_LIST_EMPTY(diff[0].tuples));
 		INSIST(ISC_LIST_EMPTY(diff[1].tuples));
-		have[0] = have[1] = ISC_FALSE;
+		have[0] = have[1] = false;
 	next: ;
 	}
 	if (itresult[0] != ISC_R_NOMORE)
@@ -2082,14 +2117,14 @@ dns_db_diffx(dns_diff_t *diff, dns_db_t *dba, dns_dbversion_t *dbvera,
 }
 
 isc_result_t
-dns_journal_compact(isc_mem_t *mctx, char *filename, isc_uint32_t serial,
-		    isc_uint32_t target_size)
+dns_journal_compact(isc_mem_t *mctx, char *filename, uint32_t serial,
+		    uint32_t target_size)
 {
 	unsigned int i;
 	journal_pos_t best_guess;
 	journal_pos_t current_pos;
-	dns_journal_t *j = NULL;
-	dns_journal_t *new = NULL;
+	dns_journal_t *j1 = NULL;
+	dns_journal_t *j2 = NULL;
 	journal_rawheader_t rawheader;
 	unsigned int copy_length;
 	size_t namelen;
@@ -2099,7 +2134,9 @@ dns_journal_compact(isc_mem_t *mctx, char *filename, isc_uint32_t serial,
 	unsigned int indexend;
 	char newname[1024];
 	char backup[1024];
-	isc_boolean_t is_backup = ISC_FALSE;
+	bool is_backup = false;
+
+	REQUIRE(filename != NULL);
 
 	namelen = strlen(filename);
 	if (namelen > 4U && strcmp(filename + namelen - 4, ".jnl") == 0)
@@ -2115,22 +2152,22 @@ dns_journal_compact(isc_mem_t *mctx, char *filename, isc_uint32_t serial,
 	if (result != ISC_R_SUCCESS)
 		return (result);
 
-	result = journal_open(mctx, filename, ISC_FALSE, ISC_FALSE, &j);
+	result = journal_open(mctx, filename, false, false, &j1);
 	if (result == ISC_R_NOTFOUND) {
-		is_backup = ISC_TRUE;
-		result = journal_open(mctx, backup, ISC_FALSE, ISC_FALSE, &j);
+		is_backup = true;
+		result = journal_open(mctx, backup, false, false, &j1);
 	}
 	if (result != ISC_R_SUCCESS)
 		return (result);
 
-	if (JOURNAL_EMPTY(&j->header)) {
-		dns_journal_destroy(&j);
+	if (JOURNAL_EMPTY(&j1->header)) {
+		dns_journal_destroy(&j1);
 		return (ISC_R_SUCCESS);
 	}
 
-	if (DNS_SERIAL_GT(j->header.begin.serial, serial) ||
-	    DNS_SERIAL_GT(serial, j->header.end.serial)) {
-		dns_journal_destroy(&j);
+	if (DNS_SERIAL_GT(j1->header.begin.serial, serial) ||
+	    DNS_SERIAL_GT(serial, j1->header.end.serial)) {
+		dns_journal_destroy(&j1);
 		return (ISC_R_RANGE);
 	}
 
@@ -2138,19 +2175,21 @@ dns_journal_compact(isc_mem_t *mctx, char *filename, isc_uint32_t serial,
 	 * Cope with very small target sizes.
 	 */
 	indexend = sizeof(journal_rawheader_t) +
-		   j->header.index_size * sizeof(journal_rawpos_t);
+		   j1->header.index_size * sizeof(journal_rawpos_t);
+	if (target_size < DNS_JOURNAL_SIZE_MIN)
+		target_size = DNS_JOURNAL_SIZE_MIN;
 	if (target_size < indexend * 2)
 		target_size = target_size/2 + indexend;
 
 	/*
 	 * See if there is any work to do.
 	 */
-	if ((isc_uint32_t) j->header.end.offset < target_size) {
-		dns_journal_destroy(&j);
+	if ((uint32_t) j1->header.end.offset < target_size) {
+		dns_journal_destroy(&j1);
 		return (ISC_R_SUCCESS);
 	}
 
-	CHECK(journal_open(mctx, newname, ISC_TRUE, ISC_TRUE, &new));
+	CHECK(journal_open(mctx, newname, true, true, &j2));
 
 	/*
 	 * Remove overhead so space test below can succeed.
@@ -2161,24 +2200,24 @@ dns_journal_compact(isc_mem_t *mctx, char *filename, isc_uint32_t serial,
 	/*
 	 * Find if we can create enough free space.
 	 */
-	best_guess = j->header.begin;
-	for (i = 0; i < j->header.index_size; i++) {
-		if (POS_VALID(j->index[i]) &&
-		    DNS_SERIAL_GE(serial, j->index[i].serial) &&
-		    ((isc_uint32_t)(j->header.end.offset - j->index[i].offset)
+	best_guess = j1->header.begin;
+	for (i = 0; i < j1->header.index_size; i++) {
+		if (POS_VALID(j1->index[i]) &&
+		    DNS_SERIAL_GE(serial, j1->index[i].serial) &&
+		    ((uint32_t)(j1->header.end.offset - j1->index[i].offset)
 		     >= target_size / 2) &&
-		    j->index[i].offset > best_guess.offset)
-			best_guess = j->index[i];
+		    j1->index[i].offset > best_guess.offset)
+			best_guess = j1->index[i];
 	}
 
 	current_pos = best_guess;
 	while (current_pos.serial != serial) {
-		CHECK(journal_next(j, &current_pos));
-		if (current_pos.serial == j->header.end.serial)
+		CHECK(journal_next(j1, &current_pos));
+		if (current_pos.serial == j1->header.end.serial)
 			break;
 
 		if (DNS_SERIAL_GE(serial, current_pos.serial) &&
-		   ((isc_uint32_t)(j->header.end.offset - current_pos.offset)
+		   ((uint32_t)(j1->header.end.offset - current_pos.offset)
 		     >= (target_size / 2)) &&
 		    current_pos.offset > best_guess.offset)
 			best_guess = current_pos;
@@ -2186,16 +2225,16 @@ dns_journal_compact(isc_mem_t *mctx, char *filename, isc_uint32_t serial,
 			break;
 	}
 
-	INSIST(best_guess.serial != j->header.end.serial);
+	INSIST(best_guess.serial != j1->header.end.serial);
 	if (best_guess.serial != serial)
-		CHECK(journal_next(j, &best_guess));
+		CHECK(journal_next(j1, &best_guess));
 
 	/*
 	 * We should now be roughly half target_size provided
 	 * we did not reach 'serial'.  If not we will just copy
 	 * all uncommitted deltas regardless of the size.
 	 */
-	copy_length = j->header.end.offset - best_guess.offset;
+	copy_length = j1->header.end.offset - best_guess.offset;
 
 	if (copy_length != 0) {
 		/*
@@ -2210,51 +2249,51 @@ dns_journal_compact(isc_mem_t *mctx, char *filename, isc_uint32_t serial,
 			goto failure;
 		}
 
-		CHECK(journal_seek(j, best_guess.offset));
-		CHECK(journal_seek(new, indexend));
+		CHECK(journal_seek(j1, best_guess.offset));
+		CHECK(journal_seek(j2, indexend));
 		for (i = 0; i < copy_length; i += size) {
 			unsigned int len = (copy_length - i) > size ? size :
 							 (copy_length - i);
-			CHECK(journal_read(j, buf, len));
-			CHECK(journal_write(new, buf, len));
+			CHECK(journal_read(j1, buf, len));
+			CHECK(journal_write(j2, buf, len));
 		}
 
-		CHECK(journal_fsync(new));
+		CHECK(journal_fsync(j2));
 
 		/*
 		 * Compute new header.
 		 */
-		new->header.begin.serial = best_guess.serial;
-		new->header.begin.offset = indexend;
-		new->header.end.serial = j->header.end.serial;
-		new->header.end.offset = indexend + copy_length;
-		new->header.sourceserial = j->header.sourceserial;
-		new->header.serialset = j->header.serialset;
+		j2->header.begin.serial = best_guess.serial;
+		j2->header.begin.offset = indexend;
+		j2->header.end.serial = j1->header.end.serial;
+		j2->header.end.offset = indexend + copy_length;
+		j2->header.sourceserial = j1->header.sourceserial;
+		j2->header.serialset = j1->header.serialset;
 
 		/*
 		 * Update the journal header.
 		 */
-		journal_header_encode(&new->header, &rawheader);
-		CHECK(journal_seek(new, 0));
-		CHECK(journal_write(new, &rawheader, sizeof(rawheader)));
-		CHECK(journal_fsync(new));
+		journal_header_encode(&j2->header, &rawheader);
+		CHECK(journal_seek(j2, 0));
+		CHECK(journal_write(j2, &rawheader, sizeof(rawheader)));
+		CHECK(journal_fsync(j2));
 
 		/*
 		 * Build new index.
 		 */
-		current_pos = new->header.begin;
-		while (current_pos.serial != new->header.end.serial) {
-			index_add(new, &current_pos);
-			CHECK(journal_next(new, &current_pos));
+		current_pos = j2->header.begin;
+		while (current_pos.serial != j2->header.end.serial) {
+			index_add(j2, &current_pos);
+			CHECK(journal_next(j2, &current_pos));
 		}
 
 		/*
 		 * Write index.
 		 */
-		CHECK(index_to_disk(new));
-		CHECK(journal_fsync(new));
+		CHECK(index_to_disk(j2));
+		CHECK(journal_fsync(j2));
 
-		indexend = new->header.end.offset;
+		indexend = j2->header.end.offset;
 		POST(indexend);
 	}
 
@@ -2262,8 +2301,8 @@ dns_journal_compact(isc_mem_t *mctx, char *filename, isc_uint32_t serial,
 	 * Close both journals before trying to rename files (this is
 	 * necessary on WIN32).
 	 */
-	dns_journal_destroy(&j);
-	dns_journal_destroy(&new);
+	dns_journal_destroy(&j1);
+	dns_journal_destroy(&j2);
 
 	/*
 	 * With a UFS file system this should just succeed and be atomic.
@@ -2300,10 +2339,10 @@ dns_journal_compact(isc_mem_t *mctx, char *filename, isc_uint32_t serial,
 	(void)isc_file_remove(newname);
 	if (buf != NULL)
 		isc_mem_put(mctx, buf, size);
-	if (j != NULL)
-		dns_journal_destroy(&j);
-	if (new != NULL)
-		dns_journal_destroy(&new);
+	if (j1 != NULL)
+		dns_journal_destroy(&j1);
+	if (j2 != NULL)
+		dns_journal_destroy(&j2);
 	return (result);
 }
 
