@@ -1,52 +1,77 @@
 /*
- * Copyright (C) 2011-2015  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
- * Permission to use, copy, modify, and/or distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
- * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
- * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
- * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
- * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
- * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
+ * See the COPYRIGHT file distributed with this work for additional
+ * information regarding copyright ownership.
  */
 
 /*! \file */
 
 #include <config.h>
 
-#include <atf-c.h>
+#if HAVE_CMOCKA
+#include <stdarg.h>
+#include <stddef.h>
+#include <setjmp.h>
 
+#include <stdbool.h>
+#include <stdlib.h>
 #include <unistd.h>
 #include <time.h>
 
+#define UNIT_TESTING
+#include <cmocka.h>
+
 #include <isc/platform.h>
 #include <isc/socket.h>
+#include <isc/task.h>
 #include <isc/print.h>
 
-#include "../task_p.h"
 #include "../unix/socket_p.h"
 #include "isctest.h"
 
-static isc_boolean_t recv_dscp;
+static bool recv_dscp;
 static unsigned int recv_dscp_value;
+static bool recv_trunc;
 
 /*
  * Helper functions
  */
 
+static int
+_setup(void **state) {
+	isc_result_t result;
+
+	UNUSED(state);
+
+	result = isc_test_begin(NULL, true, 0);
+	assert_int_equal(result, ISC_R_SUCCESS);
+
+	return (0);
+}
+
+static int
+_teardown(void **state) {
+	UNUSED(state);
+
+	isc_test_end();
+
+	return (0);
+}
+
 typedef struct {
-	isc_boolean_t done;
+	bool done;
 	isc_result_t result;
 	isc_socket_t *socket;
 } completion_t;
 
 static void
 completion_init(completion_t *completion) {
-	completion->done = ISC_FALSE;
+	completion->done = false;
 	completion->socket = NULL;
 }
 
@@ -58,28 +83,42 @@ accept_done(isc_task_t *task, isc_event_t *event) {
 	UNUSED(task);
 
 	completion->result = nevent->result;
-	completion->done = ISC_TRUE;
-	if (completion->result == ISC_R_SUCCESS)
+	completion->done = true;
+	if (completion->result == ISC_R_SUCCESS) {
 		completion->socket = nevent->newsocket;
+	}
 
 	isc_event_free(&event);
 }
 
 static void
 event_done(isc_task_t *task, isc_event_t *event) {
-	isc_socketevent_t *dev;
+	isc_socketevent_t *sev = NULL;
+	isc_socket_connev_t *connev = NULL;
 	completion_t *completion = event->ev_arg;
-
 	UNUSED(task);
 
-	dev = (isc_socketevent_t *) event;
-	completion->result = dev->result;
-	completion->done = ISC_TRUE;
-	if ((dev->attributes & ISC_SOCKEVENTATTR_DSCP) != 0) {
-		recv_dscp = ISC_TRUE;
-		recv_dscp_value = dev->dscp;;
-	} else
-		recv_dscp = ISC_FALSE;
+	switch (event->ev_type) {
+	case ISC_SOCKEVENT_RECVDONE:
+	case ISC_SOCKEVENT_SENDDONE:
+		sev = (isc_socketevent_t *) event;
+		completion->result = sev->result;
+		if ((sev->attributes & ISC_SOCKEVENTATTR_DSCP) != 0) {
+			recv_dscp = true;
+			recv_dscp_value = sev->dscp;;
+		} else {
+			recv_dscp = false;
+		}
+		recv_trunc = ((sev->attributes & ISC_SOCKEVENTATTR_TRUNC) != 0);
+		break;
+	case ISC_SOCKEVENT_CONNECT:
+		connev = (isc_socket_connev_t *) event;
+		completion->result = connev->result;
+		break;
+	default:
+		assert_false(true);
+	}
+	completion->done = true;
 	isc_event_free(&event);
 }
 
@@ -93,23 +132,11 @@ waitfor(completion_t *completion) {
 #endif
 		isc_test_nap(1000);
 	}
-	if (completion->done)
+	if (completion->done) {
 		return (ISC_R_SUCCESS);
-	return (ISC_R_FAILURE);
-}
-
-#if 0
-static isc_result_t
-waitfor(completion_t *completion) {
-	int i = 0;
-	while (!completion->done && i++ < 5000) {
-		waitbody();
 	}
-	if (completion->done)
-		return (ISC_R_SUCCESS);
 	return (ISC_R_FAILURE);
 }
-#endif
 
 static void
 waitbody(void) {
@@ -136,8 +163,9 @@ waitfor2(completion_t *c1, completion_t *c2) {
 	while (!(c1->done && c2->done) && i++ < 5000) {
 		waitbody();
 	}
-	if (c1->done && c2->done)
+	if (c1->done && c2->done) {
 		return (ISC_R_SUCCESS);
+	}
 	return (ISC_R_FAILURE);
 }
 
@@ -146,11 +174,8 @@ waitfor2(completion_t *c1, completion_t *c2) {
  */
 
 /* Test UDP sendto/recv (IPv4) */
-ATF_TC(udp_sendto);
-ATF_TC_HEAD(udp_sendto, tc) {
-	atf_tc_set_md_var(tc, "descr", "UDP sendto/recv");
-}
-ATF_TC_BODY(udp_sendto, tc) {
+static void
+udp_sendto_test(void **state)  {
 	isc_result_t result;
 	isc_sockaddr_t addr1, addr2;
 	struct in_addr in;
@@ -160,72 +185,63 @@ ATF_TC_BODY(udp_sendto, tc) {
 	completion_t completion;
 	isc_region_t r;
 
-	UNUSED(tc);
-
-	result = isc_test_begin(NULL, ISC_TRUE);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
+	UNUSED(state);
 
 	in.s_addr = inet_addr("127.0.0.1");
 	isc_sockaddr_fromin(&addr1, &in, 0);
 	isc_sockaddr_fromin(&addr2, &in, 0);
 
 	result = isc_socket_create(socketmgr, PF_INET, isc_sockettype_udp, &s1);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 	result = isc_socket_bind(s1, &addr1, 0);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 	result = isc_socket_getsockname(s1, &addr1);
-	ATF_CHECK_EQ_MSG(result, ISC_R_SUCCESS, "%s",
-			 isc_result_totext(result));
-	ATF_REQUIRE(isc_sockaddr_getport(&addr1) != 0);
+	assert_int_equal(result, ISC_R_SUCCESS);
+	assert_true(isc_sockaddr_getport(&addr1) != 0);
 
 	result = isc_socket_create(socketmgr, PF_INET, isc_sockettype_udp, &s2);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 	result = isc_socket_bind(s2, &addr2, 0);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 	result = isc_socket_getsockname(s2, &addr2);
-	ATF_CHECK_EQ_MSG(result, ISC_R_SUCCESS, "%s",
-			 isc_result_totext(result));
-	ATF_REQUIRE(isc_sockaddr_getport(&addr2) != 0);
+	assert_int_equal(result, ISC_R_SUCCESS);
+	assert_true(isc_sockaddr_getport(&addr2) != 0);
 
 	result = isc_task_create(taskmgr, 0, &task);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 
-	strcpy(sendbuf, "Hello");
+	snprintf(sendbuf, sizeof(sendbuf), "Hello");
 	r.base = (void *) sendbuf;
 	r.length = strlen(sendbuf) + 1;
 
 	completion_init(&completion);
 	result = isc_socket_sendto(s1, &r, task, event_done, &completion,
 				   &addr2, NULL);
-	ATF_CHECK_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 	waitfor(&completion);
-	ATF_CHECK(completion.done);
-	ATF_CHECK_EQ(completion.result, ISC_R_SUCCESS);
+	assert_true(completion.done);
+	assert_int_equal(completion.result, ISC_R_SUCCESS);
 
 	r.base = (void *) recvbuf;
 	r.length = BUFSIZ;
 	completion_init(&completion);
 	result = isc_socket_recv(s2, &r, 1, task, event_done, &completion);
-	ATF_CHECK_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 	waitfor(&completion);
-	ATF_CHECK(completion.done);
-	ATF_CHECK_EQ(completion.result, ISC_R_SUCCESS);
-	ATF_CHECK_STREQ(recvbuf, "Hello");
+	assert_true(completion.done);
+	assert_int_equal(completion.result, ISC_R_SUCCESS);
+	assert_string_equal(recvbuf, "Hello");
 
 	isc_task_detach(&task);
 
 	isc_socket_detach(&s1);
 	isc_socket_detach(&s2);
 
-	isc_test_end();
 }
 
 /* Test UDP sendto/recv with duplicated socket */
-ATF_TC(udp_dup);
-ATF_TC_HEAD(udp_dup, tc) {
-	atf_tc_set_md_var(tc, "descr", "duplicated socket sendto/recv");
-}
-ATF_TC_BODY(udp_dup, tc) {
+static void
+udp_dup_test(void **state) {
 	isc_result_t result;
 	isc_sockaddr_t addr1, addr2;
 	struct in_addr in;
@@ -235,82 +251,77 @@ ATF_TC_BODY(udp_dup, tc) {
 	completion_t completion;
 	isc_region_t r;
 
-	UNUSED(tc);
-
-	result = isc_test_begin(NULL, ISC_TRUE);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
+	UNUSED(state);
 
 	in.s_addr = inet_addr("127.0.0.1");
 	isc_sockaddr_fromin(&addr1, &in, 0);
 	isc_sockaddr_fromin(&addr2, &in, 0);
 
 	result = isc_socket_create(socketmgr, PF_INET, isc_sockettype_udp, &s1);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 	result = isc_socket_bind(s1, &addr1, 0);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 	result = isc_socket_getsockname(s1, &addr1);
-	ATF_CHECK_EQ_MSG(result, ISC_R_SUCCESS, "%s",
-			 isc_result_totext(result));
-	ATF_REQUIRE(isc_sockaddr_getport(&addr1) != 0);
+	assert_int_equal(result, ISC_R_SUCCESS);
+	assert_true(isc_sockaddr_getport(&addr1) != 0);
 
 	result = isc_socket_create(socketmgr, PF_INET, isc_sockettype_udp, &s2);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 	result = isc_socket_bind(s2, &addr2, 0);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 	result = isc_socket_getsockname(s2, &addr2);
-	ATF_CHECK_EQ_MSG(result, ISC_R_SUCCESS, "%s",
-			 isc_result_totext(result));
-	ATF_REQUIRE(isc_sockaddr_getport(&addr2) != 0);
+	assert_int_equal(result, ISC_R_SUCCESS);
+	assert_true(isc_sockaddr_getport(&addr2) != 0);
 
 	result = isc_socket_dup(s2, &s3);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 
 	result = isc_task_create(taskmgr, 0, &task);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 
-	strcpy(sendbuf, "Hello");
+	snprintf(sendbuf, sizeof(sendbuf), "Hello");
 	r.base = (void *) sendbuf;
 	r.length = strlen(sendbuf) + 1;
 
 	completion_init(&completion);
 	result = isc_socket_sendto(s1, &r, task, event_done, &completion,
 				   &addr2, NULL);
-	ATF_CHECK_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 	waitfor(&completion);
-	ATF_CHECK(completion.done);
-	ATF_CHECK_EQ(completion.result, ISC_R_SUCCESS);
+	assert_true(completion.done);
+	assert_int_equal(completion.result, ISC_R_SUCCESS);
 
-	strcpy(sendbuf, "World");
+	snprintf(sendbuf, sizeof(sendbuf), "World");
 	r.base = (void *) sendbuf;
 	r.length = strlen(sendbuf) + 1;
 
 	completion_init(&completion);
 	result = isc_socket_sendto(s1, &r, task, event_done, &completion,
 				   &addr2, NULL);
-	ATF_CHECK_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 	waitfor(&completion);
-	ATF_CHECK(completion.done);
-	ATF_CHECK_EQ(completion.result, ISC_R_SUCCESS);
+	assert_true(completion.done);
+	assert_int_equal(completion.result, ISC_R_SUCCESS);
 
 	r.base = (void *) recvbuf;
 	r.length = BUFSIZ;
 	completion_init(&completion);
 	result = isc_socket_recv(s2, &r, 1, task, event_done, &completion);
-	ATF_CHECK_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 	waitfor(&completion);
-	ATF_CHECK(completion.done);
-	ATF_CHECK_EQ(completion.result, ISC_R_SUCCESS);
-	ATF_CHECK_STREQ(recvbuf, "Hello");
+	assert_true(completion.done);
+	assert_int_equal(completion.result, ISC_R_SUCCESS);
+	assert_string_equal(recvbuf, "Hello");
 
 	r.base = (void *) recvbuf;
 	r.length = BUFSIZ;
 	completion_init(&completion);
 	result = isc_socket_recv(s3, &r, 1, task, event_done, &completion);
-	ATF_CHECK_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 	waitfor(&completion);
-	ATF_CHECK(completion.done);
-	ATF_CHECK_EQ(completion.result, ISC_R_SUCCESS);
-	ATF_CHECK_STREQ(recvbuf, "World");
+	assert_true(completion.done);
+	assert_int_equal(completion.result, ISC_R_SUCCESS);
+	assert_string_equal(recvbuf, "World");
 
 	isc_task_detach(&task);
 
@@ -318,15 +329,11 @@ ATF_TC_BODY(udp_dup, tc) {
 	isc_socket_detach(&s2);
 	isc_socket_detach(&s3);
 
-	isc_test_end();
 }
 
-/* Test TCP sendto/recv (IPv4) */
-ATF_TC(udp_dscp_v4);
-ATF_TC_HEAD(udp_dscp_v4, tc) {
-	atf_tc_set_md_var(tc, "descr", "UDP DSCP IPV4");
-}
-ATF_TC_BODY(udp_dscp_v4, tc) {
+/* Test UDP sendto/recv (IPv4) */
+static void
+udp_dscp_v4_test(void **state) {
 	isc_result_t result;
 	isc_sockaddr_t addr1, addr2;
 	struct in_addr in;
@@ -337,42 +344,33 @@ ATF_TC_BODY(udp_dscp_v4, tc) {
 	isc_region_t r;
 	isc_socketevent_t *socketevent;
 
-	UNUSED(tc);
+	UNUSED(state);
 
-	result = isc_test_begin(NULL, ISC_TRUE);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
 
 	in.s_addr = inet_addr("127.0.0.1");
 	isc_sockaddr_fromin(&addr1, &in, 0);
 	isc_sockaddr_fromin(&addr2, &in, 0);
 
 	result = isc_socket_create(socketmgr, PF_INET, isc_sockettype_udp, &s1);
-	ATF_CHECK_EQ_MSG(result, ISC_R_SUCCESS, "%s",
-			   isc_result_totext(result));
+	assert_int_equal(result, ISC_R_SUCCESS);
 	result = isc_socket_bind(s1, &addr1, ISC_SOCKET_REUSEADDRESS);
-	ATF_CHECK_EQ_MSG(result, ISC_R_SUCCESS, "%s",
-			   isc_result_totext(result));
+	assert_int_equal(result, ISC_R_SUCCESS);
 	result = isc_socket_getsockname(s1, &addr1);
-	ATF_CHECK_EQ_MSG(result, ISC_R_SUCCESS, "%s",
-			 isc_result_totext(result));
-	ATF_REQUIRE(isc_sockaddr_getport(&addr1) != 0);
+	assert_int_equal(result, ISC_R_SUCCESS);
+	assert_true(isc_sockaddr_getport(&addr1) != 0);
 
 	result = isc_socket_create(socketmgr, PF_INET, isc_sockettype_udp, &s2);
-	ATF_CHECK_EQ_MSG(result, ISC_R_SUCCESS, "%s",
-			   isc_result_totext(result));
+	assert_int_equal(result, ISC_R_SUCCESS);
 	result = isc_socket_bind(s2, &addr2, ISC_SOCKET_REUSEADDRESS);
-	ATF_CHECK_EQ_MSG(result, ISC_R_SUCCESS, "%s",
-			   isc_result_totext(result));
+	assert_int_equal(result, ISC_R_SUCCESS);
 	result = isc_socket_getsockname(s2, &addr2);
-	ATF_CHECK_EQ_MSG(result, ISC_R_SUCCESS, "%s",
-			 isc_result_totext(result));
-	ATF_REQUIRE(isc_sockaddr_getport(&addr2) != 0);
+	assert_int_equal(result, ISC_R_SUCCESS);
+	assert_true(isc_sockaddr_getport(&addr2) != 0);
 
 	result = isc_task_create(taskmgr, 0, &task);
-	ATF_CHECK_EQ_MSG(result, ISC_R_SUCCESS, "%s",
-			   isc_result_totext(result));
+	assert_int_equal(result, ISC_R_SUCCESS);
 
-	strcpy(sendbuf, "Hello");
+	snprintf(sendbuf, sizeof(sendbuf), "Hello");
 	r.base = (void *) sendbuf;
 	r.length = strlen(sendbuf) + 1;
 
@@ -380,7 +378,7 @@ ATF_TC_BODY(udp_dscp_v4, tc) {
 
 	socketevent = isc_socket_socketevent(mctx, s1, ISC_SOCKEVENT_SENDDONE,
 					     event_done, &completion);
-	ATF_REQUIRE(socketevent != NULL);
+	assert_non_null(socketevent);
 
 	if ((isc_net_probedscp() & ISC_NET_DSCPPKTV4) != 0) {
 		socketevent->dscp = 056; /* EF */
@@ -391,46 +389,42 @@ ATF_TC_BODY(udp_dscp_v4, tc) {
 		socketevent->attributes &= ~ISC_SOCKEVENTATTR_DSCP;
 	}
 
-	recv_dscp = ISC_FALSE;
+	recv_dscp = false;
 	recv_dscp_value = 0;
 
 	result = isc_socket_sendto2(s1, &r, task, &addr2, NULL, socketevent, 0);
-	ATF_CHECK_EQ_MSG(result, ISC_R_SUCCESS, "%s",
-			   isc_result_totext(result));
+	assert_int_equal(result, ISC_R_SUCCESS);
 	waitfor(&completion);
-	ATF_CHECK(completion.done);
-	ATF_CHECK_EQ(completion.result, ISC_R_SUCCESS);
+	assert_true(completion.done);
+	assert_int_equal(completion.result, ISC_R_SUCCESS);
 
 	r.base = (void *) recvbuf;
 	r.length = BUFSIZ;
 	completion_init(&completion);
 	result = isc_socket_recv(s2, &r, 1, task, event_done, &completion);
-	ATF_CHECK_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 	waitfor(&completion);
-	ATF_CHECK(completion.done);
-	ATF_CHECK_EQ(completion.result, ISC_R_SUCCESS);
-	ATF_CHECK_STREQ(recvbuf, "Hello");
+	assert_true(completion.done);
+	assert_int_equal(completion.result, ISC_R_SUCCESS);
+	assert_string_equal(recvbuf, "Hello");
 
 	if ((isc_net_probedscp() & ISC_NET_DSCPRECVV4) != 0) {
-		ATF_CHECK(recv_dscp);
-		ATF_CHECK_EQ(recv_dscp_value, 056);
-	} else
-		ATF_CHECK(!recv_dscp);
+		assert_true(recv_dscp);
+		assert_int_equal(recv_dscp_value, 056);
+	} else {
+		assert_false(recv_dscp);
+	}
 	isc_task_detach(&task);
 
 	isc_socket_detach(&s1);
 	isc_socket_detach(&s2);
 
-	isc_test_end();
 }
 
-/* Test TCP sendto/recv (IPv4) */
-ATF_TC(udp_dscp_v6);
-ATF_TC_HEAD(udp_dscp_v6, tc) {
-	atf_tc_set_md_var(tc, "descr", "udp dscp ipv6");
-}
-ATF_TC_BODY(udp_dscp_v6, tc) {
 #if defined(ISC_PLATFORM_HAVEIPV6) && defined(WANT_IPV6)
+/* Test UDP sendto/recv (IPv6) */
+static void
+udp_dscp_v6_test(void **state) {
 	isc_result_t result;
 	isc_sockaddr_t addr1, addr2;
 	struct in6_addr in6;
@@ -442,45 +436,36 @@ ATF_TC_BODY(udp_dscp_v6, tc) {
 	isc_socketevent_t *socketevent;
 	int n;
 
-	UNUSED(tc);
+	UNUSED(state);
 
-	result = isc_test_begin(NULL, ISC_TRUE);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
 
 	n = inet_pton(AF_INET6, "::1", &in6.s6_addr);
-	ATF_REQUIRE(n == 1);
+	assert_true(n == 1);
 	isc_sockaddr_fromin6(&addr1, &in6, 0);
 	isc_sockaddr_fromin6(&addr2, &in6, 0);
 
 	result = isc_socket_create(socketmgr, PF_INET6, isc_sockettype_udp,
 				   &s1);
-	ATF_CHECK_EQ_MSG(result, ISC_R_SUCCESS, "%s",
-			 isc_result_totext(result));
+	assert_int_equal(result, ISC_R_SUCCESS);
 	result = isc_socket_bind(s1, &addr1, 0);
-	ATF_CHECK_EQ_MSG(result, ISC_R_SUCCESS, "%s",
-			 isc_result_totext(result));
+	assert_int_equal(result, ISC_R_SUCCESS);
 	result = isc_socket_getsockname(s1, &addr1);
-	ATF_CHECK_EQ_MSG(result, ISC_R_SUCCESS, "%s",
-			 isc_result_totext(result));
-	ATF_REQUIRE(isc_sockaddr_getport(&addr1) != 0);
+	assert_int_equal(result, ISC_R_SUCCESS);
+	assert_true(isc_sockaddr_getport(&addr1) != 0);
 
 	result = isc_socket_create(socketmgr, PF_INET6, isc_sockettype_udp,
 				   &s2);
-	ATF_CHECK_EQ_MSG(result, ISC_R_SUCCESS, "%s",
-			 isc_result_totext(result));
+	assert_int_equal(result, ISC_R_SUCCESS);
 	result = isc_socket_bind(s2, &addr2, 0);
-	ATF_CHECK_EQ_MSG(result, ISC_R_SUCCESS, "%s",
-			 isc_result_totext(result));
+	assert_int_equal(result, ISC_R_SUCCESS);
 	result = isc_socket_getsockname(s2, &addr2);
-	ATF_CHECK_EQ_MSG(result, ISC_R_SUCCESS, "%s",
-			 isc_result_totext(result));
-	ATF_REQUIRE(isc_sockaddr_getport(&addr2) != 0);
+	assert_int_equal(result, ISC_R_SUCCESS);
+	assert_true(isc_sockaddr_getport(&addr2) != 0);
 
 	result = isc_task_create(taskmgr, 0, &task);
-	ATF_CHECK_EQ_MSG(result, ISC_R_SUCCESS, "%s",
-			 isc_result_totext(result));
+	assert_int_equal(result, ISC_R_SUCCESS);
 
-	strcpy(sendbuf, "Hello");
+	snprintf(sendbuf, sizeof(sendbuf), "Hello");
 	r.base = (void *) sendbuf;
 	r.length = strlen(sendbuf) + 1;
 
@@ -488,56 +473,51 @@ ATF_TC_BODY(udp_dscp_v6, tc) {
 
 	socketevent = isc_socket_socketevent(mctx, s1, ISC_SOCKEVENT_SENDDONE,
 					     event_done, &completion);
-	ATF_REQUIRE(socketevent != NULL);
+	assert_non_null(socketevent);
 
 	if ((isc_net_probedscp() & ISC_NET_DSCPPKTV6) != 0) {
 		socketevent->dscp = 056; /* EF */
 		socketevent->attributes = ISC_SOCKEVENTATTR_DSCP;
-	} else if ((isc_net_probedscp() & ISC_NET_DSCPSETV6) != 0)
+	} else if ((isc_net_probedscp() & ISC_NET_DSCPSETV6) != 0) {
 		isc_socket_dscp(s1, 056);  /* EF */
+	}
 
-	recv_dscp = ISC_FALSE;
+	recv_dscp = false;
 	recv_dscp_value = 0;
 
 	result = isc_socket_sendto2(s1, &r, task, &addr2, NULL, socketevent, 0);
-	ATF_CHECK_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 	waitfor(&completion);
-	ATF_CHECK(completion.done);
-	ATF_CHECK_EQ(completion.result, ISC_R_SUCCESS);
+	assert_true(completion.done);
+	assert_int_equal(completion.result, ISC_R_SUCCESS);
 
 	r.base = (void *) recvbuf;
 	r.length = BUFSIZ;
 	completion_init(&completion);
 	result = isc_socket_recv(s2, &r, 1, task, event_done, &completion);
-	ATF_CHECK_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 	waitfor(&completion);
-	ATF_CHECK(completion.done);
-	ATF_CHECK_EQ(completion.result, ISC_R_SUCCESS);
-	ATF_CHECK_STREQ(recvbuf, "Hello");
+	assert_true(completion.done);
+	assert_int_equal(completion.result, ISC_R_SUCCESS);
+	assert_string_equal(recvbuf, "Hello");
 	if ((isc_net_probedscp() & ISC_NET_DSCPRECVV6) != 0) {
-		ATF_CHECK(recv_dscp);
-		ATF_CHECK_EQ(recv_dscp_value, 056);
-	} else
-		ATF_CHECK(!recv_dscp);
+		assert_true(recv_dscp);
+		assert_int_equal(recv_dscp_value, 056);
+	} else {
+		assert_false(recv_dscp);
+	}
 
 	isc_task_detach(&task);
 
 	isc_socket_detach(&s1);
 	isc_socket_detach(&s2);
 
-	isc_test_end();
-#else
-	UNUSED(tc);
-	atf_tc_skip("IPv6 not available");
-#endif
 }
+#endif
 
 /* Test TCP sendto/recv (IPv4) */
-ATF_TC(tcp_dscp_v4);
-ATF_TC_HEAD(tcp_dscp_v4, tc) {
-	atf_tc_set_md_var(tc, "descr", "tcp dscp ipv4");
-}
-ATF_TC_BODY(tcp_dscp_v4, tc) {
+static void
+tcp_dscp_v4_test(void **state) {
 	isc_result_t result;
 	isc_sockaddr_t addr1;
 	struct in_addr in;
@@ -547,79 +527,78 @@ ATF_TC_BODY(tcp_dscp_v4, tc) {
 	completion_t completion, completion2;
 	isc_region_t r;
 
-	UNUSED(tc);
+	UNUSED(state);
 
-	result = isc_test_begin(NULL, ISC_TRUE);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
 
 	in.s_addr = inet_addr("127.0.0.1");
 	isc_sockaddr_fromin(&addr1, &in, 0);
 
 	result = isc_socket_create(socketmgr, PF_INET, isc_sockettype_tcp, &s1);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 
 	result = isc_socket_bind(s1, &addr1, 0);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 	result = isc_socket_getsockname(s1, &addr1);
-	ATF_CHECK_EQ_MSG(result, ISC_R_SUCCESS, "%s",
-			 isc_result_totext(result));
-	ATF_REQUIRE(isc_sockaddr_getport(&addr1) != 0);
+	assert_int_equal(result, ISC_R_SUCCESS);
+	assert_true(isc_sockaddr_getport(&addr1) != 0);
 
 	result = isc_socket_listen(s1, 3);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 
 	result = isc_socket_create(socketmgr, PF_INET, isc_sockettype_tcp, &s2);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 
 	result = isc_task_create(taskmgr, 0, &task);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 
 	completion_init(&completion2);
 	result = isc_socket_accept(s1, task, accept_done, &completion2);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 
 	completion_init(&completion);
 	result = isc_socket_connect(s2, &addr1, task, event_done, &completion);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 	waitfor2(&completion, &completion2);
-	ATF_CHECK(completion.done);
-	ATF_CHECK_EQ(completion.result, ISC_R_SUCCESS);
-	ATF_CHECK(completion2.done);
-	ATF_CHECK_EQ(completion2.result, ISC_R_SUCCESS);
+	assert_true(completion.done);
+	assert_int_equal(completion.result, ISC_R_SUCCESS);
+	assert_true(completion2.done);
+	assert_int_equal(completion2.result, ISC_R_SUCCESS);
 	s3 = completion2.socket;
 
 	isc_socket_dscp(s2, 056);  /* EF */
 
-	strcpy(sendbuf, "Hello");
+	snprintf(sendbuf, sizeof(sendbuf), "Hello");
 	r.base = (void *) sendbuf;
 	r.length = strlen(sendbuf) + 1;
 
-	recv_dscp = ISC_FALSE;
+	recv_dscp = false;
 	recv_dscp_value = 0;
 
 	completion_init(&completion);
 	result = isc_socket_sendto(s2, &r, task, event_done, &completion,
 				   NULL, NULL);
-	ATF_CHECK_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 	waitfor(&completion);
-	ATF_CHECK(completion.done);
-	ATF_CHECK_EQ(completion.result, ISC_R_SUCCESS);
+	assert_true(completion.done);
+	assert_int_equal(completion.result, ISC_R_SUCCESS);
 
 	r.base = (void *) recvbuf;
 	r.length = BUFSIZ;
 	completion_init(&completion);
 	result = isc_socket_recv(s3, &r, 1, task, event_done, &completion);
-	ATF_CHECK_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 	waitfor(&completion);
-	ATF_CHECK(completion.done);
-	ATF_CHECK_EQ(completion.result, ISC_R_SUCCESS);
-	ATF_CHECK_STREQ(recvbuf, "Hello");
+	assert_true(completion.done);
+	assert_int_equal(completion.result, ISC_R_SUCCESS);
+	assert_string_equal(recvbuf, "Hello");
 
 	if ((isc_net_probedscp() & ISC_NET_DSCPRECVV4) != 0) {
-		if (recv_dscp)
-			ATF_CHECK_EQ(recv_dscp_value, 056);
-	} else
-		ATF_CHECK(!recv_dscp);
+		if (recv_dscp) {
+			assert_int_equal(recv_dscp_value, 056);
+		}
+	} else {
+		assert_false(recv_dscp);
+	}
 
 	isc_task_detach(&task);
 
@@ -627,16 +606,12 @@ ATF_TC_BODY(tcp_dscp_v4, tc) {
 	isc_socket_detach(&s2);
 	isc_socket_detach(&s3);
 
-	isc_test_end();
 }
 
+#if defined(ISC_PLATFORM_HAVEIPV6) && defined(WANT_IPV6)
 /* Test TCP sendto/recv (IPv6) */
-ATF_TC(tcp_dscp_v6);
-ATF_TC_HEAD(tcp_dscp_v6, tc) {
-	atf_tc_set_md_var(tc, "descr", "tcp dscp ipv6");
-}
-ATF_TC_BODY(tcp_dscp_v6, tc) {
-#ifdef ISC_PLATFORM_HAVEIPV6
+static void
+tcp_dscp_v6_test(void **state) {
 	isc_result_t result;
 	isc_sockaddr_t addr1;
 	struct in6_addr in6;
@@ -647,86 +622,85 @@ ATF_TC_BODY(tcp_dscp_v6, tc) {
 	isc_region_t r;
 	int n;
 
-	UNUSED(tc);
+	UNUSED(state);
 
-	result = isc_test_begin(NULL, ISC_TRUE);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
 
 	n = inet_pton(AF_INET6, "::1", &in6.s6_addr);
-	ATF_REQUIRE(n == 1);
+	assert_true(n == 1);
 	isc_sockaddr_fromin6(&addr1, &in6, 0);
 
 	result = isc_socket_create(socketmgr, PF_INET6, isc_sockettype_tcp,
 				   &s1);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 
 	result = isc_socket_bind(s1, &addr1, 0);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 	result = isc_socket_getsockname(s1, &addr1);
-	ATF_CHECK_EQ_MSG(result, ISC_R_SUCCESS, "%s",
-			 isc_result_totext(result));
-	ATF_REQUIRE(isc_sockaddr_getport(&addr1) != 0);
+	assert_int_equal(result, ISC_R_SUCCESS);
+	assert_true(isc_sockaddr_getport(&addr1) != 0);
 
 	result = isc_socket_listen(s1, 3);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 
 	result = isc_socket_create(socketmgr, PF_INET6, isc_sockettype_tcp,
 				   &s2);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 
 	result = isc_task_create(taskmgr, 0, &task);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 
 	completion_init(&completion2);
 	result = isc_socket_accept(s1, task, accept_done, &completion2);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 
 	completion_init(&completion);
 	result = isc_socket_connect(s2, &addr1, task, event_done, &completion);
-	ATF_REQUIRE_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 	waitfor2(&completion, &completion2);
-	ATF_CHECK(completion.done);
-	ATF_CHECK_EQ(completion.result, ISC_R_SUCCESS);
-	ATF_CHECK(completion2.done);
-	ATF_CHECK_EQ(completion2.result, ISC_R_SUCCESS);
+	assert_true(completion.done);
+	assert_int_equal(completion.result, ISC_R_SUCCESS);
+	assert_true(completion2.done);
+	assert_int_equal(completion2.result, ISC_R_SUCCESS);
 	s3 = completion2.socket;
 
 	isc_socket_dscp(s2, 056);  /* EF */
 
-	strcpy(sendbuf, "Hello");
+	snprintf(sendbuf, sizeof(sendbuf), "Hello");
 	r.base = (void *) sendbuf;
 	r.length = strlen(sendbuf) + 1;
 
-	recv_dscp = ISC_FALSE;
+	recv_dscp = false;
 	recv_dscp_value = 0;
 
 	completion_init(&completion);
 	result = isc_socket_sendto(s2, &r, task, event_done, &completion,
 				   NULL, NULL);
-	ATF_CHECK_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 	waitfor(&completion);
-	ATF_CHECK(completion.done);
-	ATF_CHECK_EQ(completion.result, ISC_R_SUCCESS);
+	assert_true(completion.done);
+	assert_int_equal(completion.result, ISC_R_SUCCESS);
 
 	r.base = (void *) recvbuf;
 	r.length = BUFSIZ;
 	completion_init(&completion);
 	result = isc_socket_recv(s3, &r, 1, task, event_done, &completion);
-	ATF_CHECK_EQ(result, ISC_R_SUCCESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
 	waitfor(&completion);
-	ATF_CHECK(completion.done);
-	ATF_CHECK_EQ(completion.result, ISC_R_SUCCESS);
-	ATF_CHECK_STREQ(recvbuf, "Hello");
+	assert_true(completion.done);
+	assert_int_equal(completion.result, ISC_R_SUCCESS);
+	assert_string_equal(recvbuf, "Hello");
 
 	if ((isc_net_probedscp() & ISC_NET_DSCPRECVV6) != 0) {
 		/*
 		 * IPV6_RECVTCLASS is undefined for TCP however
 		 * if we do get it it should be the value we set.
 		 */
-		if (recv_dscp)
-			ATF_CHECK_EQ(recv_dscp_value, 056);
-	} else
-		ATF_CHECK(!recv_dscp);
+		if (recv_dscp) {
+			assert_int_equal(recv_dscp_value, 056);
+		}
+	} else {
+		assert_false(recv_dscp);
+	}
 
 	isc_task_detach(&task);
 
@@ -734,40 +708,28 @@ ATF_TC_BODY(tcp_dscp_v6, tc) {
 	isc_socket_detach(&s2);
 	isc_socket_detach(&s3);
 
-	isc_test_end();
-#else
-	UNUSED(tc);
-	atf_tc_skip("IPv6 not available");
+}
 #endif
-}
 
-ATF_TC(net_probedscp);
-ATF_TC_HEAD(net_probedscp, tc) {
-	atf_tc_set_md_var(tc, "descr", "probe dscp capabilities");
-}
-ATF_TC_BODY(net_probedscp, tc) {
+/* probe dscp capabilities */
+static void
+net_probedscp_test(void **state) {
 	unsigned int n;
 
-	UNUSED(tc);
+	UNUSED(state);
 
 	n = isc_net_probedscp();
-	ATF_CHECK((n & ~ISC_NET_DSCPALL) == 0);
+	assert_true((n & ~ISC_NET_DSCPALL) == 0);
 
 	/* ISC_NET_DSCPSETV4 MUST be set if any is set. */
-	if (n & (ISC_NET_DSCPSETV4|ISC_NET_DSCPPKTV4|ISC_NET_DSCPRECVV4))
-		ATF_CHECK_MSG((n & ISC_NET_DSCPSETV4) != 0,
-			      "IPv4:%s%s%s\n",
-			      (n & ISC_NET_DSCPSETV4) ? " set" : " none",
-			      (n & ISC_NET_DSCPPKTV4) ? " packet" : "",
-			      (n & ISC_NET_DSCPRECVV4) ? " receive" : "");
+	if (n & (ISC_NET_DSCPPKTV4|ISC_NET_DSCPRECVV4)) {
+		assert_true((n & ISC_NET_DSCPSETV4) != 0);
+	}
 
 	/* ISC_NET_DSCPSETV6 MUST be set if any is set. */
-	if (n & (ISC_NET_DSCPSETV6|ISC_NET_DSCPPKTV6|ISC_NET_DSCPRECVV6))
-		ATF_CHECK_MSG((n & ISC_NET_DSCPSETV6) != 0,
-			      "IPv6:%s%s%s\n",
-			      (n & ISC_NET_DSCPSETV6) ? " set" : " none",
-			      (n & ISC_NET_DSCPPKTV6) ? " packet" : "",
-			      (n & ISC_NET_DSCPRECVV6) ? " receive" : "");
+	if (n & (ISC_NET_DSCPPKTV6|ISC_NET_DSCPRECVV6)) {
+		assert_true((n & ISC_NET_DSCPSETV6) != 0);
+	}
 
 #if 0
 	fprintf(stdout, "IPv4:%s%s%s\n",
@@ -782,17 +744,154 @@ ATF_TC_BODY(net_probedscp, tc) {
 #endif
 }
 
+/* Test UDP truncation detection */
+static void
+udp_trunc_test(void **state) {
+	isc_result_t result;
+	isc_sockaddr_t addr1, addr2;
+	struct in_addr in;
+	isc_socket_t *s1 = NULL, *s2 = NULL;
+	isc_task_t *task = NULL;
+	char sendbuf[BUFSIZ*2], recvbuf[BUFSIZ];
+	completion_t completion;
+	isc_region_t r;
+	isc_socketevent_t *socketevent;
+
+	UNUSED(state);
+
+	in.s_addr = inet_addr("127.0.0.1");
+	isc_sockaddr_fromin(&addr1, &in, 0);
+	isc_sockaddr_fromin(&addr2, &in, 0);
+
+	result = isc_socket_create(socketmgr, PF_INET, isc_sockettype_udp, &s1);
+	assert_int_equal(result, ISC_R_SUCCESS);
+	result = isc_socket_bind(s1, &addr1, ISC_SOCKET_REUSEADDRESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
+	result = isc_socket_getsockname(s1, &addr1);
+	assert_int_equal(result, ISC_R_SUCCESS);
+	assert_true(isc_sockaddr_getport(&addr1) != 0);
+	result = isc_socket_create(socketmgr, PF_INET, isc_sockettype_udp, &s2);
+	assert_int_equal(result, ISC_R_SUCCESS);
+	result = isc_socket_bind(s2, &addr2, ISC_SOCKET_REUSEADDRESS);
+	assert_int_equal(result, ISC_R_SUCCESS);
+	result = isc_socket_getsockname(s2, &addr2);
+	assert_int_equal(result, ISC_R_SUCCESS);
+	assert_true(isc_sockaddr_getport(&addr2) != 0);
+
+	result = isc_task_create(taskmgr, 0, &task);
+	assert_int_equal(result, ISC_R_SUCCESS);
+
+	/*
+	 * Send a message that will not be truncated.
+	 */
+	memset(sendbuf, 0xff, sizeof(sendbuf));
+	snprintf(sendbuf, sizeof(sendbuf), "Hello");
+	r.base = (void *) sendbuf;
+	r.length = strlen(sendbuf) + 1;
+
+	completion_init(&completion);
+
+	socketevent = isc_socket_socketevent(mctx, s1, ISC_SOCKEVENT_SENDDONE,
+					     event_done, &completion);
+	assert_non_null(socketevent);
+
+	result = isc_socket_sendto2(s1, &r, task, &addr2, NULL, socketevent, 0);
+	assert_int_equal(result, ISC_R_SUCCESS);
+	waitfor(&completion);
+	assert_true(completion.done);
+	assert_int_equal(completion.result, ISC_R_SUCCESS);
+
+	r.base = (void *) recvbuf;
+	r.length = BUFSIZ;
+	completion_init(&completion);
+	recv_trunc = false;
+	result = isc_socket_recv(s2, &r, 1, task, event_done, &completion);
+	assert_int_equal(result, ISC_R_SUCCESS);
+	waitfor(&completion);
+	assert_true(completion.done);
+	assert_int_equal(completion.result, ISC_R_SUCCESS);
+	assert_string_equal(recvbuf, "Hello");
+	assert_false(recv_trunc);
+
+	/*
+	 * Send a message that will be truncated.
+	 */
+	memset(sendbuf, 0xff, sizeof(sendbuf));
+	snprintf(sendbuf, sizeof(sendbuf), "Hello");
+	r.base = (void *) sendbuf;
+	r.length = sizeof(sendbuf);
+
+	completion_init(&completion);
+
+	socketevent = isc_socket_socketevent(mctx, s1, ISC_SOCKEVENT_SENDDONE,
+					     event_done, &completion);
+	assert_non_null(socketevent);
+
+	result = isc_socket_sendto2(s1, &r, task, &addr2, NULL, socketevent, 0);
+	assert_int_equal(result, ISC_R_SUCCESS);
+	waitfor(&completion);
+	assert_true(completion.done);
+	assert_int_equal(completion.result, ISC_R_SUCCESS);
+
+	r.base = (void *) recvbuf;
+	r.length = BUFSIZ;
+	completion_init(&completion);
+	recv_trunc = false;
+	result = isc_socket_recv(s2, &r, 1, task, event_done, &completion);
+	assert_int_equal(result, ISC_R_SUCCESS);
+	waitfor(&completion);
+	assert_true(completion.done);
+	assert_int_equal(completion.result, ISC_R_SUCCESS);
+	assert_string_equal(recvbuf, "Hello");
+	assert_true(recv_trunc);
+
+	isc_task_detach(&task);
+
+	isc_socket_detach(&s1);
+	isc_socket_detach(&s2);
+
+}
+
 /*
  * Main
  */
-ATF_TP_ADD_TCS(tp) {
-	ATF_TP_ADD_TC(tp, udp_sendto);
-	ATF_TP_ADD_TC(tp, udp_dup);
-	ATF_TP_ADD_TC(tp, tcp_dscp_v4);
-	ATF_TP_ADD_TC(tp, tcp_dscp_v6);
-	ATF_TP_ADD_TC(tp, udp_dscp_v4);
-	ATF_TP_ADD_TC(tp, udp_dscp_v6);
-	ATF_TP_ADD_TC(tp, net_probedscp);
+int
+main(void) {
+	const struct CMUnitTest tests[] = {
+		cmocka_unit_test_setup_teardown(udp_sendto_test,
+						_setup, _teardown),
+		cmocka_unit_test_setup_teardown(udp_dup_test,
+						_setup, _teardown),
+		cmocka_unit_test_setup_teardown(tcp_dscp_v4_test,
+						_setup, _teardown),
+#if defined(ISC_PLATFORM_HAVEIPV6) && defined(WANT_IPV6)
+		cmocka_unit_test_setup_teardown(tcp_dscp_v6_test,
+						_setup, _teardown),
+#endif
+		cmocka_unit_test_setup_teardown(udp_dscp_v4_test,
+						_setup, _teardown),
+#if defined(ISC_PLATFORM_HAVEIPV6) && defined(WANT_IPV6)
+		cmocka_unit_test_setup_teardown(udp_dscp_v6_test,
+						_setup, _teardown),
+#endif
+		cmocka_unit_test_setup_teardown(net_probedscp_test,
+						_setup, _teardown),
+		cmocka_unit_test_setup_teardown(udp_trunc_test,
+						_setup, _teardown),
+	};
 
-	return (atf_no_error());
+	return (cmocka_run_group_tests(tests, NULL, NULL));
 }
+
+#else /* HAVE_CMOCKA */
+
+#include <stdio.h>
+
+int
+main(void) {
+	printf("1..0 # Skipped: cmocka not available\n");
+	return (0);
+
+}
+
+#endif
